@@ -134,7 +134,7 @@ if check_version $AUTOCONF $autoconf_vers_num $autoconf_vers ; then
     check_version $AUTOHEADER $autoconf_vers_num $autoconf_vers autoconf
 fi
 if check_version $AUTOMAKE $automake_vers_num $automake_vers; then
-  check_version $ACLOCAL $automake_vers_num $autoconf_vers automake
+    check_version $ACLOCAL $automake_vers_num $automake_vers automake
 fi
 #if check_version $GETTEXT $gettext_vers_num $gettext_vers; then
 #  check_version $MSGMERGE $gettext_vers_num $gettext_vers gettext
@@ -159,4 +159,6 @@ $AUTOMAKE --gnu;
 echo "Running autoconf${FORCE} ..."
 $AUTOCONF${FORCE}
 
-echo "You may now run \"./configure --enable-maintainer-mode && make\"."
+echo "You may now run:
+  ./configure --enable-maintainer-mode && make
+"
