@@ -14,8 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -31,16 +30,10 @@
 
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
+#include <sys/types.h>
 #include <assert.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
-#ifdef HAVE_LOCALE_H
+#include <unistd.h>
 #include <locale.h>
-#endif
 #include <errno.h>
 
 #include "gpgme.h"
@@ -97,7 +90,6 @@ struct engine_llass
     int gpg_agent:1;  /* Assume this is a gpg-agent connection.  */
   } opt;
 
-  char request_origin[10];  /* Copy from the CTX.  */
 };
 typedef struct engine_llass *engine_llass_t;
 
@@ -133,15 +125,14 @@ llass_get_home_dir (void)
 static char *
 llass_get_version (const char *file_name)
 {
-  (void)file_name;
-  return NULL;
+  return strdup ("1.0");
 }
 
 
 static const char *
 llass_get_req_version (void)
 {
-  return NULL;
+  return "1.0";
 }
 
 
@@ -213,17 +204,13 @@ llass_release (void *engine)
 
 
 /* Create a new instance. If HOME_DIR is NULL standard options for use
-   with gpg-agent are issued.  */
+   with gpg-agent are issued.  */  
 static gpgme_error_t
-llass_new (void **engine, const char *file_name, const char *home_dir,
-           const char *version)
+llass_new (void **engine, const char *file_name, const char *home_dir)
 {
   gpgme_error_t err = 0;
   engine_llass_t llass;
   char *optstr;
-  char *env_tty = NULL;
-
-  (void)version; /* Not yet used.  */
 
   llass = calloc (1, sizeof *llass);
   if (!llass)
@@ -241,7 +228,7 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
       /* Very simple parser only working for the one option we support.  */
       /* Note that wk promised to write a regression test if this
          parser will be extended.  */
-      if (!strncmp (home_dir, "GPG_AGENT", 9)
+      if (!strncmp (home_dir, "GPG_AGENT", 9) 
           && (!home_dir[9] || home_dir[9] == ' '))
         llass->opt.gpg_agent = 1;
     }
@@ -252,7 +239,6 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
   if (err)
     goto leave;
   assuan_ctx_set_system_hooks (llass->assuan_ctx, &_gpgme_assuan_system_hooks);
-  assuan_set_flag (llass->assuan_ctx, ASSUAN_CONVEY_COMMENTS, 1);
 
   err = assuan_socket_connect (llass->assuan_ctx, file_name, 0, 0);
   if (err)
@@ -267,7 +253,7 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
         goto leave;
       if (dft_display)
         {
-          if (gpgrt_asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
+          if (asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
             {
               err = gpg_error_from_syserror ();
               free (dft_display);
@@ -277,43 +263,34 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
 
           err = assuan_transact (llass->assuan_ctx, optstr, NULL, NULL, NULL,
                                  NULL, NULL, NULL);
-          gpgrt_free (optstr);
+          free (optstr);
           if (err)
             goto leave;
         }
     }
 
-  if (llass->opt.gpg_agent)
-    err = _gpgme_getenv ("GPG_TTY", &env_tty);
-
-  if (llass->opt.gpg_agent && (isatty (1) || env_tty || err))
+  if (llass->opt.gpg_agent && isatty (1))
     {
-      int rc = 0;
+      int rc;
       char dft_ttyname[64];
       char *dft_ttytype = NULL;
 
-      if (err)
-        goto leave;
-      else if (env_tty)
-        {
-          snprintf (dft_ttyname, sizeof (dft_ttyname), "%s", env_tty);
-          free (env_tty);
-        }
-      else
-        rc = ttyname_r (1, dft_ttyname, sizeof (dft_ttyname));
-
-      /* Even though isatty() returns 1, ttyname_r() may fail in many
-	 ways, e.g., when /dev/pts is not accessible under chroot.  */
-      if (!rc)
+      rc = ttyname_r (1, dft_ttyname, sizeof (dft_ttyname));
+      if (rc)
 	{
-	  if (gpgrt_asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
+	  err = gpg_error_from_errno (rc);
+	  goto leave;
+	}
+      else
+	{
+	  if (asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
 	    {
 	      err = gpg_error_from_syserror ();
 	      goto leave;
 	    }
 	  err = assuan_transact (llass->assuan_ctx, optstr, NULL, NULL, NULL,
 				 NULL, NULL, NULL);
-	  gpgrt_free (optstr);
+	  free (optstr);
 	  if (err)
             goto leave;
 
@@ -322,17 +299,17 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
 	    goto leave;
 	  if (dft_ttytype)
 	    {
-	      if (gpgrt_asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype)< 0)
+	      if (asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype) < 0)
 		{
 		  err = gpg_error_from_syserror ();
 		  free (dft_ttytype);
 		  goto leave;
 		}
 	      free (dft_ttytype);
-
+              
 	      err = assuan_transact (llass->assuan_ctx, optstr, NULL, NULL,
 				     NULL, NULL, NULL, NULL);
-	      gpgrt_free (optstr);
+	      free (optstr);
 	      if (err)
                 goto leave;
 	    }
@@ -367,31 +344,13 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
 }
 
 
-/* Copy flags from CTX into the engine object.  */
-static void
-llass_set_engine_flags (void *engine, const gpgme_ctx_t ctx)
-{
-  engine_llass_t llass = engine;
-
-  if (ctx->request_origin)
-    {
-      if (strlen (ctx->request_origin) + 1 > sizeof llass->request_origin)
-        strcpy (llass->request_origin, "xxx"); /* Too long  - force error */
-      else
-        strcpy (llass->request_origin, ctx->request_origin);
-    }
-  else
-    *llass->request_origin = 0;
-}
-
-
 static gpgme_error_t
 llass_set_locale (void *engine, int category, const char *value)
 {
   gpgme_error_t err;
   engine_llass_t llass = engine;
   char *optstr;
-  const char *catstr;
+  char *catstr;
 
   if (!llass->opt.gpg_agent)
     return 0;
@@ -399,10 +358,7 @@ llass_set_locale (void *engine, int category, const char *value)
   /* FIXME: If value is NULL, we need to reset the option to default.
      But we can't do this.  So we error out here.  gpg-agent needs
      support for this.  */
-  if (0)
-    ;
-#ifdef LC_CTYPE
-  else if (category == LC_CTYPE)
+  if (category == LC_CTYPE)
     {
       catstr = "lc-ctype";
       if (!value && llass->lc_ctype_set)
@@ -410,7 +366,6 @@ llass_set_locale (void *engine, int category, const char *value)
       if (value)
 	llass->lc_ctype_set = 1;
     }
-#endif
 #ifdef LC_MESSAGES
   else if (category == LC_MESSAGES)
     {
@@ -428,13 +383,13 @@ llass_set_locale (void *engine, int category, const char *value)
   if (!value)
     return 0;
 
-  if (gpgrt_asprintf (&optstr, "OPTION %s=%s", catstr, value) < 0)
-    err = gpg_error_from_syserror ();
+  if (asprintf (&optstr, "OPTION %s=%s", catstr, value) < 0)
+    err = gpg_error_from_errno (errno);
   else
     {
       err = assuan_transact (llass->assuan_ctx, optstr, NULL, NULL,
 			     NULL, NULL, NULL, NULL);
-      gpgrt_free (optstr);
+      free (optstr);
     }
   return err;
 }
@@ -445,10 +400,7 @@ llass_set_locale (void *engine, int category, const char *value)
 static gpgme_error_t
 inquire_cb (engine_llass_t llass, const char *keyword, const char *args)
 {
-  gpg_error_t err, err2;
-  gpgme_data_t data = NULL;
-  char buf[1024];
-  gpgme_ssize_t n;
+  gpg_error_t err;
 
   if (llass->opt.gpg_agent && !strcmp (keyword, "PINENTRY_LAUNCHED"))
     {
@@ -457,23 +409,17 @@ inquire_cb (engine_llass_t llass, const char *keyword, const char *args)
 
   if (llass->user.inq_cb)
     {
+      gpgme_data_t data = NULL;
+
       err = llass->user.inq_cb (llass->user.inq_cb_value,
                                 keyword, args, &data);
       if (!err && data)
         {
-          while ((n = gpgme_data_read (data, buf, sizeof buf)) > 0)
-            {
-              err = assuan_send_data (llass->assuan_ctx, buf, n);
-              if (err)
-                break;
-            }
-          /* Tell the caller that we are finished with the data
-           * object.  The error code from assuan_send_data has
-           * priority over the one from the cleanup function. */
-          err2 = llass->user.inq_cb (llass->user.inq_cb_value,
-                                     NULL, NULL, &data);
-          if (!err)
-            err = err2;
+          /* FIXME: Returning data is not yet implemented.  However we
+             need to allow the caller to cleanup his data object.
+             Thus we run the callback in finish mode immediately.  */
+          err = llass->user.inq_cb (llass->user.inq_cb_value,
+                                    NULL, NULL, &data);
         }
     }
   else
@@ -502,13 +448,13 @@ llass_status_handler (void *opaque, int fd)
 	     case, we are done for now.  */
 	  if (gpg_err_code (err) == GPG_ERR_EAGAIN)
 	    {
-	      TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+	      TRACE1 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		      "fd 0x%x: EAGAIN reading assuan line (ignored)", fd);
 	      err = 0;
 	      continue;
 	    }
-
-	  TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+	  
+	  TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: error reading assuan line: %s",
                   fd, gpg_strerror (err));
 	}
@@ -539,7 +485,7 @@ llass_status_handler (void *opaque, int fd)
             err = llass->user.data_cb (llass->user.data_cb_value,
                                        src, linelen);
 
-          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: D inlinedata; status from cb: %s",
                   fd, (llass->user.data_cb ?
                        (err? gpg_strerror (err):"ok"):"no callback"));
@@ -552,7 +498,7 @@ llass_status_handler (void *opaque, int fd)
           if (llass->user.data_cb)
             err = llass->user.data_cb (llass->user.data_cb_value, NULL, 0);
 
-          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: END line; status from cb: %s",
                   fd, (llass->user.data_cb ?
                        (err? gpg_strerror (err):"ok"):"no callback"));
@@ -578,7 +524,7 @@ llass_status_handler (void *opaque, int fd)
             err = llass->user.status_cb (llass->user.status_cb_value,
                                          src, args);
 
-          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE3 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: S line (%s) - status from cb: %s",
                   fd, line+2, (llass->user.status_cb ?
                                (err? gpg_strerror (err):"ok"):"no callback"));
@@ -605,7 +551,7 @@ llass_status_handler (void *opaque, int fd)
             args++;
 
           err = inquire_cb (llass, src, args);
-          if (!err)
+          if (!err) 
             {
               /* Flush and send END.  */
               err = assuan_send_data (llass->assuan_ctx, NULL, 0);
@@ -624,7 +570,7 @@ llass_status_handler (void *opaque, int fd)
 	    err = atoi (line+4);
 	  else
 	    err = gpg_error (GPG_ERR_GENERAL);
-          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: ERR line: %s",
                   fd, err ? gpg_strerror (err) : "ok");
 
@@ -641,7 +587,7 @@ llass_status_handler (void *opaque, int fd)
 	       && line[0] == 'O' && line[1] == 'K'
 	       && (line[2] == '\0' || line[2] == ' '))
 	{
-          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE1 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: OK line", fd);
 
 	  llass->last_op_err = 0;
@@ -666,8 +612,8 @@ add_io_cb (engine_llass_t llass, iocb_data_t *iocbd, gpgme_io_cb_t handler)
 {
   gpgme_error_t err;
 
-  TRACE_BEG  (DEBUG_ENGINE, "engine-assuan:add_io_cb", llass,
-              "fd=%d, dir %d", iocbd->fd, iocbd->dir);
+  TRACE_BEG2 (DEBUG_ENGINE, "engine-assuan:add_io_cb", llass,
+              "fd %d, dir %d", iocbd->fd, iocbd->dir);
   err = (*llass->io_cbs.add) (llass->io_cbs.add_priv,
 			      iocbd->fd, iocbd->dir,
 			      handler, iocbd->data, &iocbd->tag);
@@ -684,36 +630,16 @@ static gpgme_error_t
 start (engine_llass_t llass, const char *command)
 {
   gpgme_error_t err;
-  assuan_fd_t afdlist[5];
   int fdlist[5];
   int nfds;
-  int i;
-
-  if (*llass->request_origin && llass->opt.gpg_agent)
-    {
-      char *cmd;
-
-      cmd = _gpgme_strconcat ("OPTION pretend-request-origin=",
-                              llass->request_origin, NULL);
-      if (!cmd)
-        return gpg_error_from_syserror ();
-      err = assuan_transact (llass->assuan_ctx, cmd, NULL, NULL, NULL,
-                             NULL, NULL, NULL);
-      free (cmd);
-      if (err && gpg_err_code (err) != GPG_ERR_UNKNOWN_OPTION)
-        return err;
-    }
 
   /* We need to know the fd used by assuan for reads.  We do this by
      using the assumption that the first returned fd from
      assuan_get_active_fds() is always this one.  */
   nfds = assuan_get_active_fds (llass->assuan_ctx, 0 /* read fds */,
-                                afdlist, DIM (afdlist));
+                                fdlist, DIM (fdlist));
   if (nfds < 1)
     return gpg_error (GPG_ERR_GENERAL);	/* FIXME */
-  /* For now... */
-  for (i = 0; i < nfds; i++)
-    fdlist[i] = (int) afdlist[i];
 
   /* We "duplicate" the file descriptor, so we can close it here (we
      can't close fdlist[0], as that is closed by libassuan, and
@@ -793,7 +719,7 @@ llass_io_event (void *engine, gpgme_event_io_t type, void *type_data)
 {
   engine_llass_t llass = engine;
 
-  TRACE (DEBUG_ENGINE, "gpgme:llass_io_event", llass,
+  TRACE3 (DEBUG_ENGINE, "gpgme:llass_io_event", llass,
           "event %p, type %d, type_data %p",
           llass->io_cbs.event, type, type_data);
   if (llass->io_cbs.event)
@@ -813,14 +739,13 @@ struct engine_ops _gpgme_engine_ops_assuan =
     /* Member functions.  */
     llass_release,
     NULL,		/* reset */
-    NULL,               /* set_status_cb */
     NULL,               /* set_status_handler */
     NULL,		/* set_command_handler */
     NULL,               /* set_colon_line_handler */
     llass_set_locale,
     NULL,		/* set_protocol */
-    llass_set_engine_flags,
     NULL,               /* decrypt */
+    NULL,               /* decrypt_verify */
     NULL,               /* delete */
     NULL,		/* edit */
     NULL,               /* encrypt */
@@ -831,24 +756,15 @@ struct engine_ops _gpgme_engine_ops_assuan =
     NULL,               /* import */
     NULL,               /* keylist */
     NULL,               /* keylist_ext */
-    NULL,               /* keylist_data */
-    NULL,               /* keysign */
-    NULL,               /* revsig */
-    NULL,               /* tofu_policy */
     NULL,               /* sign */
+    NULL,		/* trustlist */
     NULL,               /* verify */
     NULL,               /* getauditlog */
-    NULL,               /* setexpire */
     llass_transact,     /* opassuan_transact */
     NULL,		/* conf_load */
     NULL,		/* conf_save */
-    NULL,		/* conf_dir */
-    NULL,               /* query_swdb */
     llass_set_io_cbs,
     llass_io_event,
     llass_cancel,
-    llass_cancel_op,
-    NULL,               /* passwd */
-    NULL,               /* set_pinentry_mode */
-    NULL                /* opspawn */
+    llass_cancel_op
   };
