@@ -1,23 +1,23 @@
 /* engine-g13.c - G13 engine.
- * Copyright (C) 2000 Werner Koch (dd9jn)
- * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2009 g10 Code GmbH
- *
- * This file is part of GPGME.
- *
- * GPGME is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * GPGME is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
- */
+   Copyright (C) 2000 Werner Koch (dd9jn)
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2009 g10 Code GmbH
+ 
+   This file is part of GPGME.
+
+   GPGME is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
+   GPGME is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -25,16 +25,10 @@
 
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
+#include <sys/types.h>
 #include <assert.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
-#ifdef HAVE_LOCALE_H
+#include <unistd.h>
 #include <locale.h>
-#endif
 #include <fcntl.h> /* FIXME */
 #include <errno.h>
 
@@ -91,7 +85,7 @@ struct engine_g13
 typedef struct engine_g13 *engine_g13_t;
 
 
-static void g13_io_event (void *engine,
+static void g13_io_event (void *engine, 
                             gpgme_event_io_t type, void *type_data);
 
 
@@ -100,14 +94,14 @@ static char *
 g13_get_version (const char *file_name)
 {
   return _gpgme_get_program_version (file_name ? file_name
-				     : _gpgme_get_default_g13_name ());
+				     : _gpgme_get_g13_path ());
 }
 
 
 static const char *
 g13_get_req_version (void)
 {
-  return "2.1.0";
+  return NEED_G13_VERSION;
 }
 
 
@@ -212,34 +206,28 @@ g13_release (void *engine)
 
 
 static gpgme_error_t
-g13_new (void **engine, const char *file_name, const char *home_dir,
-         const char *version)
+g13_new (void **engine, const char *file_name, const char *home_dir)
 {
   gpgme_error_t err = 0;
   engine_g13_t g13;
-  const char *pgmname;
   int argc;
-  const char *argv[5];
+  char *argv[5];
   char *dft_display = NULL;
   char dft_ttyname[64];
-  char *env_tty = NULL;
   char *dft_ttytype = NULL;
   char *optstr;
 
-  (void)version; /* Not yet used.  */
-
   g13 = calloc (1, sizeof *g13);
   if (!g13)
-    return gpg_error_from_syserror ();
+    return gpg_error_from_errno (errno);
 
   g13->status_cb.fd = -1;
   g13->status_cb.dir = 1;
   g13->status_cb.tag = 0;
   g13->status_cb.data = g13;
 
-  pgmname = file_name ? file_name : _gpgme_get_default_g13_name ();
   argc = 0;
-  argv[argc++] = _gpgme_get_basename (pgmname);
+  argv[argc++] = "g13";
   if (home_dir)
     {
       argv[argc++] = "--homedir";
@@ -256,11 +244,13 @@ g13_new (void **engine, const char *file_name, const char *home_dir,
   assuan_ctx_set_system_hooks (g13->assuan_ctx, &_gpgme_assuan_system_hooks);
 
 #if USE_DESCRIPTOR_PASSING
-  err = assuan_pipe_connect (g13->assuan_ctx, pgmname, argv,
-                             NULL, NULL, NULL, ASSUAN_PIPE_CONNECT_FDPASSING);
+  err = assuan_pipe_connect
+    (g13->assuan_ctx, file_name ? file_name : _gpgme_get_g13_path (),
+     argv, NULL, NULL, NULL, ASSUAN_PIPE_CONNECT_FDPASSING);
 #else
-  err = assuan_pipe_connect (g13->assuan_ctx, pgmname, argv,
-                             NULL, NULL, NULL, 0);
+  err = assuan_pipe_connect
+    (g13->assuan_ctx, file_name ? file_name : _gpgme_get_g13_path (),
+     argv, NULL, NULL, NULL, 0);
 #endif
   if (err)
     goto leave;
@@ -270,48 +260,41 @@ g13_new (void **engine, const char *file_name, const char *home_dir,
     goto leave;
   if (dft_display)
     {
-      if (gpgrt_asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
+      if (asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
         {
 	  free (dft_display);
-	  err = gpg_error_from_syserror ();
+	  err = gpg_error_from_errno (errno);
 	  goto leave;
 	}
       free (dft_display);
 
       err = assuan_transact (g13->assuan_ctx, optstr, NULL, NULL, NULL,
 			     NULL, NULL, NULL);
-      gpgrt_free (optstr);
+      free (optstr);
       if (err)
 	goto leave;
     }
 
-  err = _gpgme_getenv ("GPG_TTY", &env_tty);
-  if (isatty (1) || env_tty || err)
+  if (isatty (1))
     {
-      int rc = 0;
+      int rc;
 
-      if (err)
-        goto leave;
-      else if (env_tty)
-        {
-          snprintf (dft_ttyname, sizeof (dft_ttyname), "%s", env_tty);
-          free (env_tty);
-        }
-      else
-        rc = ttyname_r (1, dft_ttyname, sizeof (dft_ttyname));
-
-      /* Even though isatty() returns 1, ttyname_r() may fail in many
-	 ways, e.g., when /dev/pts is not accessible under chroot.  */
-      if (!rc)
+      rc = ttyname_r (1, dft_ttyname, sizeof (dft_ttyname));
+      if (rc)
 	{
-	  if (gpgrt_asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
+	  err = gpg_error_from_errno (rc);
+	  goto leave;
+	}
+      else
+	{
+	  if (asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
 	    {
-	      err = gpg_error_from_syserror ();
+	      err = gpg_error_from_errno (errno);
 	      goto leave;
 	    }
 	  err = assuan_transact (g13->assuan_ctx, optstr, NULL, NULL, NULL,
 				 NULL, NULL, NULL);
-	  gpgrt_free (optstr);
+	  free (optstr);
 	  if (err)
 	    goto leave;
 
@@ -320,17 +303,17 @@ g13_new (void **engine, const char *file_name, const char *home_dir,
 	    goto leave;
 	  if (dft_ttytype)
 	    {
-	      if (gpgrt_asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype)< 0)
+	      if (asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype) < 0)
 		{
 		  free (dft_ttytype);
-		  err = gpg_error_from_syserror ();
+		  err = gpg_error_from_errno (errno);
 		  goto leave;
 		}
 	      free (dft_ttytype);
 
 	      err = assuan_transact (g13->assuan_ctx, optstr, NULL, NULL,
 				     NULL, NULL, NULL, NULL);
-	      gpgrt_free (optstr);
+	      free (optstr);
 	      if (err)
 		goto leave;
 	    }
@@ -366,15 +349,12 @@ g13_set_locale (void *engine, int category, const char *value)
   engine_g13_t g13 = engine;
   gpgme_error_t err;
   char *optstr;
-  const char *catstr;
+  char *catstr;
 
   /* FIXME: If value is NULL, we need to reset the option to default.
      But we can't do this.  So we error out here.  G13 needs support
      for this.  */
-  if (0)
-    ;
-#ifdef LC_CTYPE
-  else if (category == LC_CTYPE)
+  if (category == LC_CTYPE)
     {
       catstr = "lc-ctype";
       if (!value && g13->lc_ctype_set)
@@ -382,7 +362,6 @@ g13_set_locale (void *engine, int category, const char *value)
       if (value)
 	g13->lc_ctype_set = 1;
     }
-#endif
 #ifdef LC_MESSAGES
   else if (category == LC_MESSAGES)
     {
@@ -397,25 +376,24 @@ g13_set_locale (void *engine, int category, const char *value)
     return gpg_error (GPG_ERR_INV_VALUE);
 
   /* FIXME: Reset value to default.  */
-  if (!value)
+  if (!value) 
     return 0;
 
-  if (gpgrt_asprintf (&optstr, "OPTION %s=%s", catstr, value) < 0)
-    err = gpg_error_from_syserror ();
+  if (asprintf (&optstr, "OPTION %s=%s", catstr, value) < 0)
+    err = gpg_error_from_errno (errno);
   else
     {
       err = assuan_transact (g13->assuan_ctx, optstr, NULL, NULL,
 			     NULL, NULL, NULL, NULL);
-      gpgrt_free (optstr);
+      free (optstr);
     }
 
   return err;
 }
 
 
-#if USE_DESCRIPTOR_PASSING
 static gpgme_error_t
-g13_assuan_simple_command (assuan_context_t ctx, const char *cmd,
+g13_assuan_simple_command (assuan_context_t ctx, char *cmd,
 			   engine_status_handler_t status_fnc,
 			   void *status_fnc_value)
 {
@@ -423,9 +401,6 @@ g13_assuan_simple_command (assuan_context_t ctx, const char *cmd,
   char *line;
   size_t linelen;
 
-  (void)status_fnc;
-  (void)status_fnc_value;
-
   err = assuan_write_line (ctx, cmd);
   if (err)
     return err;
@@ -467,7 +442,6 @@ g13_assuan_simple_command (assuan_context_t ctx, const char *cmd,
 
   return err;
 }
-#endif
 
 
 static gpgme_error_t
@@ -486,7 +460,7 @@ status_handler (void *opaque, int fd)
 	{
 	  /* Try our best to terminate the connection friendly.  */
 	  /*	  assuan_write_line (g13->assuan_ctx, "BYE"); */
-          TRACE (DEBUG_CTX, "gpgme:status_handler", g13,
+          TRACE2 (DEBUG_CTX, "gpgme:status_handler", g13,
 		  "fd 0x%x: error reading assuan line: %s",
                   fd, gpg_strerror (err));
 	}
@@ -498,10 +472,10 @@ status_handler (void *opaque, int fd)
 	    err = atoi (&line[4]);
 	  if (! err)
 	    err = gpg_error (GPG_ERR_GENERAL);
-          TRACE (DEBUG_CTX, "gpgme:status_handler", g13,
+          TRACE2 (DEBUG_CTX, "gpgme:status_handler", g13,
 		  "fd 0x%x: ERR line: %s",
                   fd, err ? gpg_strerror (err) : "ok");
-
+	  
 	  /* Command execution errors are not fatal, as we use
 	     a session based protocol.  */
 	  data->op_err = err;
@@ -514,7 +488,7 @@ status_handler (void *opaque, int fd)
 	       && line[0] == 'O' && line[1] == 'K'
 	       && (line[2] == '\0' || line[2] == ' '))
 	{
-          TRACE (DEBUG_CTX, "gpgme:status_handler", g13,
+          TRACE1 (DEBUG_CTX, "gpgme:status_handler", g13,
 		  "fd 0x%x: OK line", fd);
 
 	  _gpgme_io_close (g13->status_cb.fd);
@@ -556,7 +530,7 @@ status_handler (void *opaque, int fd)
           else
             err = 0;
 
-          TRACE (DEBUG_CTX, "gpgme:g13_status_handler", g13,
+          TRACE2 (DEBUG_CTX, "gpgme:g13_status_handler", g13,
 		  "fd 0x%x: D inlinedata; status from cb: %s",
                   fd, (g13->user.data_cb ?
                        (err? gpg_strerror (err):"ok"):"no callback"));
@@ -571,7 +545,7 @@ status_handler (void *opaque, int fd)
 	  src = line + 2;
           while (*src == ' ')
             src++;
-
+	  
 	  args = strchr (line + 2, ' ');
 	  if (!args)
 	    args = line + linelen; /* set to an empty string */
@@ -587,7 +561,7 @@ status_handler (void *opaque, int fd)
           else
             err = 0;
 
-          TRACE (DEBUG_CTX, "gpgme:g13_status_handler", g13,
+          TRACE3 (DEBUG_CTX, "gpgme:g13_status_handler", g13,
 		  "fd 0x%x: S line (%s) - status from cb: %s",
                   fd, line+2, (g13->user.status_cb ?
                                (err? gpg_strerror (err):"ok"):"no callback"));
@@ -595,12 +569,12 @@ status_handler (void *opaque, int fd)
       else if (linelen >= 7
                && line[0] == 'I' && line[1] == 'N' && line[2] == 'Q'
                && line[3] == 'U' && line[4] == 'I' && line[5] == 'R'
-               && line[6] == 'E'
+               && line[6] == 'E' 
                && (line[7] == '\0' || line[7] == ' '))
         {
           char *src;
 	  char *args;
-
+	  
           for (src=line+7; *src == ' '; src++)
             ;
 
@@ -614,7 +588,7 @@ status_handler (void *opaque, int fd)
             args++;
 
           err = default_inq_cb (g13, src, args);
-          if (!err)
+          if (!err) 
             {
               /* Flush and send END.  */
               err = assuan_send_data (g13->assuan_ctx, NULL, 0);
@@ -628,18 +602,18 @@ status_handler (void *opaque, int fd)
         }
     }
   while (!err && assuan_pending_line (g13->assuan_ctx));
-
+  
   return err;
 }
-
+  
 
 static gpgme_error_t
 add_io_cb (engine_g13_t g13, iocb_data_t *iocbd, gpgme_io_cb_t handler)
 {
   gpgme_error_t err;
 
-  TRACE_BEG  (DEBUG_ENGINE, "engine-g13:add_io_cb", g13,
-              "fd=%d, dir %d", iocbd->fd, iocbd->dir);
+  TRACE_BEG2 (DEBUG_ENGINE, "engine-g13:add_io_cb", g13,
+              "fd %d, dir %d", iocbd->fd, iocbd->dir);
   err = (*g13->io_cbs.add) (g13->io_cbs.add_priv,
 			      iocbd->fd, iocbd->dir,
 			      handler, iocbd->data, &iocbd->tag);
@@ -656,21 +630,16 @@ static gpgme_error_t
 start (engine_g13_t g13, const char *command)
 {
   gpgme_error_t err;
-  assuan_fd_t afdlist[5];
   int fdlist[5];
   int nfds;
-  int i;
 
   /* We need to know the fd used by assuan for reads.  We do this by
      using the assumption that the first returned fd from
      assuan_get_active_fds() is always this one.  */
   nfds = assuan_get_active_fds (g13->assuan_ctx, 0 /* read fds */,
-                                afdlist, DIM (afdlist));
+                                fdlist, DIM (fdlist));
   if (nfds < 1)
     return gpg_error (GPG_ERR_GENERAL);	/* FIXME */
-  /* For now... */
-  for (i = 0; i < nfds; i++)
-    fdlist[i] = (int) afdlist[i];
 
   /* We "duplicate" the file descriptor, so we can close it here (we
      can't close fdlist[0], as that is closed by libassuan, and
@@ -760,7 +729,7 @@ g13_io_event (void *engine, gpgme_event_io_t type, void *type_data)
 {
   engine_g13_t g13 = engine;
 
-  TRACE (DEBUG_ENGINE, "gpgme:g13_io_event", g13,
+  TRACE3 (DEBUG_ENGINE, "gpgme:g13_io_event", g13,
           "event %p, type %d, type_data %p",
           g13->io_cbs.event, type, type_data);
   if (g13->io_cbs.event)
@@ -771,7 +740,7 @@ g13_io_event (void *engine, gpgme_event_io_t type, void *type_data)
 struct engine_ops _gpgme_engine_ops_g13 =
   {
     /* Static functions.  */
-    _gpgme_get_default_g13_name,
+    _gpgme_get_g13_path,
     NULL,
     g13_get_version,
     g13_get_req_version,
@@ -784,14 +753,13 @@ struct engine_ops _gpgme_engine_ops_g13 =
 #else
     NULL,			/* reset */
 #endif
-    NULL,               /* set_status_cb */
     NULL,               /* set_status_handler */
     NULL,		/* set_command_handler */
     NULL,               /* set_colon_line_handler */
     g13_set_locale,
     NULL,		/* set_protocol */
-    NULL,               /* set_engine_flags */
     NULL,               /* decrypt */
+    NULL,               /* decrypt_verify */
     NULL,               /* delete */
     NULL,		/* edit */
     NULL,               /* encrypt */
@@ -802,26 +770,15 @@ struct engine_ops _gpgme_engine_ops_g13 =
     NULL,               /* import */
     NULL,               /* keylist */
     NULL,               /* keylist_ext */
-    NULL,               /* keylist_data */
-    NULL,               /* keysign */
-    NULL,               /* revsig */
-    NULL,               /* tofu_policy */
     NULL,               /* sign */
+    NULL,		/* trustlist */
     NULL,               /* verify */
     NULL,               /* getauditlog */
-    NULL,               /* setexpire */
-    NULL,               /* setownertrust */
     g13_transact,
-    NULL,               /* getdirect */
     NULL,		/* conf_load */
     NULL,		/* conf_save */
-    NULL,		/* conf_dir */
-    NULL,               /* query_swdb */
     g13_set_io_cbs,
     g13_io_event,
     g13_cancel,
     g13_cancel_op,
-    NULL,               /* passwd */
-    NULL,               /* set_pinentry_mode */
-    NULL                /* opspawn */
   };
