@@ -1,23 +1,23 @@
 /* engine-uiserver.c - Uiserver engine.
- * Copyright (C) 2000 Werner Koch (dd9jn)
- * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2009 g10 Code GmbH
- *
- * This file is part of GPGME.
- *
- * GPGME is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * GPGME is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
- */
+   Copyright (C) 2000 Werner Koch (dd9jn)
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2009 g10 Code GmbH
+ 
+   This file is part of GPGME.
+
+   GPGME is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
+   GPGME is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 /* Peculiar: Use special keys from email address for recipient and
    signer (==sender).  Use no data objects with encryption for
@@ -29,13 +29,9 @@
 
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
+#include <sys/types.h>
 #include <assert.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#include <unistd.h>
 #include <locale.h>
 #include <fcntl.h> /* FIXME */
 #include <errno.h>
@@ -49,6 +45,7 @@
 #include "data.h"
 
 #include "assuan.h"
+#include "status-table.h"
 #include "debug.h"
 
 #include "engine-backend.h"
@@ -90,8 +87,6 @@ struct engine_uiserver
   {
     engine_status_handler_t fnc;
     void *fnc_value;
-    gpgme_status_cb_t mon_cb;
-    void *mon_cb_value;
   } status;
 
   struct
@@ -105,7 +100,7 @@ struct engine_uiserver
       int linelen;
     } attic;
     int any; /* any data line seen */
-  } colon;
+  } colon; 
 
   gpgme_data_t inline_data;  /* Used to collect D lines.  */
 
@@ -115,7 +110,7 @@ struct engine_uiserver
 typedef struct engine_uiserver *engine_uiserver_t;
 
 
-static void uiserver_io_event (void *engine,
+static void uiserver_io_event (void *engine, 
                             gpgme_event_io_t type, void *type_data);
 
 
@@ -123,15 +118,14 @@ static void uiserver_io_event (void *engine,
 static char *
 uiserver_get_version (const char *file_name)
 {
-  (void)file_name;
-  return NULL;
+  return strdup ("1.0");
 }
 
 
 static const char *
 uiserver_get_req_version (void)
 {
-  return NULL;
+  return "1.0";
 }
 
 
@@ -187,8 +181,6 @@ close_notify_handler (int fd, void *opaque)
 static gpgme_error_t
 default_inq_cb (engine_uiserver_t uiserver, const char *line)
 {
-  (void)uiserver;
-
   if (!strncmp (line, "PINENTRY_LAUNCHED", 17) && (line[17]==' '||!line[17]))
     {
       _gpgme_allow_set_foreground_window ((pid_t)strtoul (line+17, NULL, 10));
@@ -241,20 +233,15 @@ uiserver_release (void *engine)
 
 
 static gpgme_error_t
-uiserver_new (void **engine, const char *file_name, const char *home_dir,
-              const char *version)
+uiserver_new (void **engine, const char *file_name, const char *home_dir)
 {
   gpgme_error_t err = 0;
   engine_uiserver_t uiserver;
   char *dft_display = NULL;
   char dft_ttyname[64];
-  char *env_tty = NULL;
   char *dft_ttytype = NULL;
   char *optstr;
 
-  (void)home_dir;
-  (void)version; /* Not yet used.  */
-
   uiserver = calloc (1, sizeof *uiserver);
   if (!uiserver)
     return gpg_error_from_syserror ();
@@ -306,7 +293,7 @@ uiserver_new (void **engine, const char *file_name, const char *home_dir,
 
   err = assuan_socket_connect (uiserver->assuan_ctx,
 			       file_name ?
-			       file_name : _gpgme_get_default_uisrv_socket (),
+			       file_name : _gpgme_get_uiserver_socket_path (),
 			       0, ASSUAN_SOCKET_SERVER_FDPASSING);
   if (err)
     goto leave;
@@ -316,48 +303,41 @@ uiserver_new (void **engine, const char *file_name, const char *home_dir,
     goto leave;
   if (dft_display)
     {
-      if (gpgrt_asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
+      if (asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
         {
-	  err = gpg_error_from_syserror ();
 	  free (dft_display);
+	  err = gpg_error_from_errno (errno);
 	  goto leave;
 	}
       free (dft_display);
 
       err = assuan_transact (uiserver->assuan_ctx, optstr, NULL, NULL, NULL,
 			     NULL, NULL, NULL);
-      gpgrt_free (optstr);
+      free (optstr);
       if (err)
 	goto leave;
     }
 
-  err = _gpgme_getenv ("GPG_TTY", &env_tty);
-  if (isatty (1) || env_tty || err)
+  if (isatty (1))
     {
-      int rc = 0;
+      int rc;
 
-      if (err)
-        goto leave;
-      else if (env_tty)
-        {
-          snprintf (dft_ttyname, sizeof (dft_ttyname), "%s", env_tty);
-          free (env_tty);
-        }
-      else
-        rc = ttyname_r (1, dft_ttyname, sizeof (dft_ttyname));
-
-      /* Even though isatty() returns 1, ttyname_r() may fail in many
-	 ways, e.g., when /dev/pts is not accessible under chroot.  */
-      if (!rc)
+      rc = ttyname_r (1, dft_ttyname, sizeof (dft_ttyname));
+      if (rc)
 	{
-	  if (gpgrt_asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
+	  err = gpg_error_from_errno (rc);
+	  goto leave;
+	}
+      else
+	{
+	  if (asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
 	    {
-	      err = gpg_error_from_syserror ();
+	      err = gpg_error_from_errno (errno);
 	      goto leave;
 	    }
 	  err = assuan_transact (uiserver->assuan_ctx, optstr, NULL, NULL, NULL,
 				 NULL, NULL, NULL);
-	  gpgrt_free (optstr);
+	  free (optstr);
 	  if (err)
 	    goto leave;
 
@@ -366,17 +346,17 @@ uiserver_new (void **engine, const char *file_name, const char *home_dir,
 	    goto leave;
 	  if (dft_ttytype)
 	    {
-	      if (gpgrt_asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype)< 0)
+	      if (asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype) < 0)
 		{
-		  err = gpg_error_from_syserror ();
 		  free (dft_ttytype);
+		  err = gpg_error_from_errno (errno);
 		  goto leave;
 		}
 	      free (dft_ttytype);
 
 	      err = assuan_transact (uiserver->assuan_ctx, optstr, NULL, NULL,
 				     NULL, NULL, NULL, NULL);
-	      gpgrt_free (optstr);
+	      free (optstr);
 	      if (err)
 		goto leave;
 	    }
@@ -411,7 +391,7 @@ uiserver_set_locale (void *engine, int category, const char *value)
   engine_uiserver_t uiserver = engine;
   gpgme_error_t err;
   char *optstr;
-  const char *catstr;
+  char *catstr;
 
   /* FIXME: If value is NULL, we need to reset the option to default.
      But we can't do this.  So we error out here.  UISERVER needs support
@@ -438,16 +418,16 @@ uiserver_set_locale (void *engine, int category, const char *value)
     return gpg_error (GPG_ERR_INV_VALUE);
 
   /* FIXME: Reset value to default.  */
-  if (!value)
+  if (!value) 
     return 0;
 
-  if (gpgrt_asprintf (&optstr, "OPTION %s=%s", catstr, value) < 0)
-    err = gpg_error_from_syserror ();
+  if (asprintf (&optstr, "OPTION %s=%s", catstr, value) < 0)
+    err = gpg_error_from_errno (errno);
   else
     {
       err = assuan_transact (uiserver->assuan_ctx, optstr, NULL, NULL,
 			     NULL, NULL, NULL, NULL);
-      gpgrt_free (optstr);
+      free (optstr);
     }
 
   return err;
@@ -469,12 +449,14 @@ uiserver_set_protocol (void *engine, gpgme_protocol_t protocol)
 }
 
 
+/* Forward declaration.  */
+static gpgme_status_code_t parse_status (const char *name);
+
 static gpgme_error_t
-uiserver_assuan_simple_command (engine_uiserver_t uiserver, const char *cmd,
-                                engine_status_handler_t status_fnc,
-                                void *status_fnc_value)
+uiserver_assuan_simple_command (assuan_context_t ctx, char *cmd,
+			     engine_status_handler_t status_fnc,
+			     void *status_fnc_value)
 {
-  assuan_context_t ctx = uiserver->assuan_ctx;
   gpg_error_t err;
   char *line;
   size_t linelen;
@@ -512,18 +494,9 @@ uiserver_assuan_simple_command (engine_uiserver_t uiserver, const char *cmd,
 	  else
 	    *(rest++) = 0;
 
-	  r = _gpgme_parse_status (line + 2);
-          if (uiserver->status.mon_cb && r != GPGME_STATUS_PROGRESS)
-            {
-              /* Note that we call the monitor even if we do
-               * not know the status code (r < 0).  */
-              err = uiserver->status.mon_cb (uiserver->status.mon_cb_value,
-                                             line + 2, rest);
-            }
+	  r = parse_status (line + 2);
 
-          if (err)
-            ;
-	  else if (r >= 0 && status_fnc)
+	  if (r >= 0 && status_fnc)
 	    err = status_fnc (status_fnc_value, r, rest);
 	  else
 	    err = gpg_error (GPG_ERR_GENERAL);
@@ -545,7 +518,7 @@ uiserver_set_fd (engine_uiserver_t uiserver, fd_type_t fd_type, const char *opt)
 {
   gpg_error_t err = 0;
   char line[COMMANDLINELEN];
-  const char *which;
+  char *which;
   iocb_data_t *iocb_data;
   int dir;
 
@@ -580,7 +553,7 @@ uiserver_set_fd (engine_uiserver_t uiserver, fd_type_t fd_type, const char *opt)
       int fds[2];
 
       if (_gpgme_io_pipe (fds, 0) < 0)
-	return gpg_error_from_syserror ();
+	return gpg_error_from_errno (errno);
 
       iocb_data->fd = dir ? fds[0] : fds[1];
       iocb_data->server_fd = dir ? fds[1] : fds[0];
@@ -605,7 +578,7 @@ uiserver_set_fd (engine_uiserver_t uiserver, fd_type_t fd_type, const char *opt)
   else
     snprintf (line, COMMANDLINELEN, "%s FD", which);
 
-  err = uiserver_assuan_simple_command (uiserver, line, NULL, NULL);
+  err = uiserver_assuan_simple_command (uiserver->assuan_ctx, line, NULL, NULL);
 
  leave_set_fd:
   if (err)
@@ -643,6 +616,27 @@ map_data_enc (gpgme_data_t d)
 }
 
 
+static int
+status_cmp (const void *ap, const void *bp)
+{
+  const struct status_table_s *a = ap;
+  const struct status_table_s *b = bp;
+
+  return strcmp (a->name, b->name);
+}
+
+
+static gpgme_status_code_t
+parse_status (const char *name)
+{
+  struct status_table_s t, *r;
+  t.name = name;
+  r = bsearch (&t, status_table, DIM(status_table) - 1,
+	       sizeof t, status_cmp);
+  return r ? r->code : -1;
+}
+
+
 static gpgme_error_t
 status_handler (void *opaque, int fd)
 {
@@ -659,7 +653,7 @@ status_handler (void *opaque, int fd)
 	{
 	  /* Try our best to terminate the connection friendly.  */
 	  /*	  assuan_write_line (uiserver->assuan_ctx, "BYE"); */
-          TRACE (DEBUG_CTX, "gpgme:status_handler", uiserver,
+          TRACE3 (DEBUG_CTX, "gpgme:status_handler", uiserver,
 		  "fd 0x%x: error from assuan (%d) getting status line : %s",
                   fd, err, gpg_strerror (err));
 	}
@@ -671,7 +665,7 @@ status_handler (void *opaque, int fd)
 	    err = atoi (&line[4]);
 	  if (! err)
 	    err = gpg_error (GPG_ERR_GENERAL);
-          TRACE (DEBUG_CTX, "gpgme:status_handler", uiserver,
+          TRACE2 (DEBUG_CTX, "gpgme:status_handler", uiserver,
 		  "fd 0x%x: ERR line - mapped to: %s",
                   fd, err ? gpg_strerror (err) : "ok");
 	  /* Try our best to terminate the connection friendly.  */
@@ -682,14 +676,9 @@ status_handler (void *opaque, int fd)
 	       && (line[2] == '\0' || line[2] == ' '))
 	{
 	  if (uiserver->status.fnc)
-            {
-              char emptystring[1] = {0};
-              err = uiserver->status.fnc (uiserver->status.fnc_value,
-                                          GPGME_STATUS_EOF, emptystring);
-              if (gpg_err_code (err) == GPG_ERR_FALSE)
-                err = 0; /* Drop special error code.  */
-            }
-
+	    err = uiserver->status.fnc (uiserver->status.fnc_value,
+				     GPGME_STATUS_EOF, "");
+	  
 	  if (!err && uiserver->colon.fnc && uiserver->colon.any)
             {
               /* We must tell a colon function about the EOF. We do
@@ -700,7 +689,7 @@ status_handler (void *opaque, int fd)
               uiserver->colon.any = 0;
               err = uiserver->colon.fnc (uiserver->colon.fnc_value, NULL);
             }
-          TRACE (DEBUG_CTX, "gpgme:status_handler", uiserver,
+          TRACE2 (DEBUG_CTX, "gpgme:status_handler", uiserver,
 		  "fd 0x%x: OK line - final status: %s",
                   fd, err ? gpg_strerror (err) : "ok");
 	  _gpgme_io_close (uiserver->status_cb.fd);
@@ -726,11 +715,11 @@ status_handler (void *opaque, int fd)
 	    {
 	      char *newline = realloc (*aline, *alinelen + linelen + 1);
 	      if (!newline)
-		err = gpg_error_from_syserror ();
+		err = gpg_error_from_errno (errno);
 	      else
 		{
 		  *aline = newline;
-		  uiserver->colon.attic.linesize = *alinelen + linelen + 1;
+		  uiserver->colon.attic.linesize += linelen + 1;
 		}
 	    }
 	  if (!err)
@@ -752,12 +741,12 @@ status_handler (void *opaque, int fd)
 		      *dst = *src++;
 		      (*alinelen)++;
 		    }
-
+		  
 		  if (*dst == '\n')
 		    {
 		      /* Terminate the pending line, pass it to the colon
 			 handler and reset it.  */
-
+		      
 		      uiserver->colon.any = 1;
 		      if (*alinelen > 1 && *(dst - 1) == '\r')
 			dst--;
@@ -775,7 +764,7 @@ status_handler (void *opaque, int fd)
 		    dst++;
 		}
 	    }
-          TRACE (DEBUG_CTX, "gpgme:status_handler", uiserver,
+          TRACE2 (DEBUG_CTX, "gpgme:status_handler", uiserver,
 		  "fd 0x%x: D line; final status: %s",
                   fd, err? gpg_strerror (err):"ok");
         }
@@ -786,7 +775,7 @@ status_handler (void *opaque, int fd)
           char *src = line + 2;
 	  char *end = line + linelen;
 	  char *dst = src;
-          gpgme_ssize_t nwritten;
+          ssize_t nwritten;
 
           linelen = 0;
           while (src < end)
@@ -800,10 +789,10 @@ status_handler (void *opaque, int fd)
                 }
               else
                 *dst++ = *src++;
-
+              
               linelen++;
             }
-
+          
           src = line + 2;
           while (linelen > 0)
             {
@@ -811,14 +800,14 @@ status_handler (void *opaque, int fd)
               if (!nwritten || (nwritten < 0 && errno != EINTR)
                   || nwritten > linelen)
                 {
-                  err = gpg_error_from_syserror ();
+                  err = gpg_error_from_errno (errno);
                   break;
                 }
               src += nwritten;
               linelen -= nwritten;
             }
 
-          TRACE (DEBUG_CTX, "gpgme:status_handler", uiserver,
+          TRACE2 (DEBUG_CTX, "gpgme:status_handler", uiserver,
 		  "fd 0x%x: D inlinedata; final status: %s",
                   fd, err? gpg_strerror (err):"ok");
         }
@@ -827,35 +816,30 @@ status_handler (void *opaque, int fd)
 	{
 	  char *rest;
 	  gpgme_status_code_t r;
-
+	  
 	  rest = strchr (line + 2, ' ');
 	  if (!rest)
 	    rest = line + linelen; /* set to an empty string */
 	  else
 	    *(rest++) = 0;
 
-	  r = _gpgme_parse_status (line + 2);
+	  r = parse_status (line + 2);
 
 	  if (r >= 0)
 	    {
 	      if (uiserver->status.fnc)
-                {
-                  err = uiserver->status.fnc (uiserver->status.fnc_value,
-                                              r, rest);
-                  if (gpg_err_code (err) == GPG_ERR_FALSE)
-                    err = 0; /* Drop special error code.  */
-                }
+		err = uiserver->status.fnc (uiserver->status.fnc_value, r, rest);
 	    }
 	  else
 	    fprintf (stderr, "[UNKNOWN STATUS]%s %s", line + 2, rest);
-          TRACE (DEBUG_CTX, "gpgme:status_handler", uiserver,
+          TRACE3 (DEBUG_CTX, "gpgme:status_handler", uiserver,
 		  "fd 0x%x: S line (%s) - final status: %s",
                   fd, line+2, err? gpg_strerror (err):"ok");
 	}
       else if (linelen >= 7
                && line[0] == 'I' && line[1] == 'N' && line[2] == 'Q'
                && line[3] == 'U' && line[4] == 'I' && line[5] == 'R'
-               && line[6] == 'E'
+               && line[6] == 'E' 
                && (line[7] == '\0' || line[7] == ' '))
         {
           char *keyword = line+7;
@@ -868,7 +852,7 @@ status_handler (void *opaque, int fd)
 
     }
   while (!err && assuan_pending_line (uiserver->assuan_ctx));
-
+	  
   return err;
 }
 
@@ -878,8 +862,8 @@ add_io_cb (engine_uiserver_t uiserver, iocb_data_t *iocbd, gpgme_io_cb_t handler
 {
   gpgme_error_t err;
 
-  TRACE_BEG  (DEBUG_ENGINE, "engine-uiserver:add_io_cb", uiserver,
-              "fd=%d, dir %d", iocbd->fd, iocbd->dir);
+  TRACE_BEG2 (DEBUG_ENGINE, "engine-uiserver:add_io_cb", uiserver,
+              "fd %d, dir %d", iocbd->fd, iocbd->dir);
   err = (*uiserver->io_cbs.add) (uiserver->io_cbs.add_priv,
 			      iocbd->fd, iocbd->dir,
 			      handler, iocbd->data, &iocbd->tag);
@@ -954,27 +938,18 @@ uiserver_reset (void *engine)
 
   /* We must send a reset because we need to reset the list of
      signers.  Note that RESET does not reset OPTION commands. */
-  return uiserver_assuan_simple_command (uiserver, "RESET", NULL, NULL);
+  return uiserver_assuan_simple_command (uiserver->assuan_ctx, "RESET", NULL, NULL);
 }
 
 
 static gpgme_error_t
-uiserver_decrypt (void *engine,
-                  gpgme_decrypt_flags_t flags,
-                  gpgme_data_t ciph, gpgme_data_t plain,
-                  int export_session_key, const char *override_session_key,
-                  int auto_key_retrieve)
+_uiserver_decrypt (void *engine, int verify,
+		   gpgme_data_t ciph, gpgme_data_t plain)
 {
   engine_uiserver_t uiserver = engine;
   gpgme_error_t err;
   const char *protocol;
   char *cmd;
-  int verify = !!(flags & GPGME_DECRYPT_VERIFY);
-
-  (void)override_session_key; /* Fixme: We need to see now to add this
-                               * to the UI server protocol  */
-  (void)auto_key_retrieve;    /* Not yet supported.  */
-
 
   if (!uiserver)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -987,38 +962,52 @@ uiserver_decrypt (void *engine,
   else
     return gpgme_error (GPG_ERR_UNSUPPORTED_PROTOCOL);
 
-  if (gpgrt_asprintf (&cmd, "DECRYPT%s%s%s", protocol,
-		verify ? "" : " --no-verify",
-                export_session_key ? " --export-session-key" : "") < 0)
-    return gpg_error_from_syserror ();
+  if (asprintf (&cmd, "DECRYPT%s%s", protocol,
+		verify ? "" : " --no-verify") < 0)
+    return gpg_error_from_errno (errno);
 
   uiserver->input_cb.data = ciph;
   err = uiserver_set_fd (uiserver, INPUT_FD,
 			 map_data_enc (uiserver->input_cb.data));
   if (err)
     {
-      gpgrt_free (cmd);
+      free (cmd);
       return gpg_error (GPG_ERR_GENERAL);	/* FIXME */
     }
   uiserver->output_cb.data = plain;
   err = uiserver_set_fd (uiserver, OUTPUT_FD, 0);
   if (err)
     {
-      gpgrt_free (cmd);
+      free (cmd);
       return gpg_error (GPG_ERR_GENERAL);	/* FIXME */
     }
   uiserver->inline_data = NULL;
 
   err = start (engine, cmd);
-  gpgrt_free (cmd);
+  free (cmd);
   return err;
 }
 
 
+static gpgme_error_t
+uiserver_decrypt (void *engine, gpgme_data_t ciph, gpgme_data_t plain)
+{
+  return _uiserver_decrypt (engine, 0, ciph, plain);
+}
+
+
+static gpgme_error_t
+uiserver_decrypt_verify (void *engine, gpgme_data_t ciph, gpgme_data_t plain)
+{
+  return _uiserver_decrypt (engine, 1, ciph, plain);
+}
+
+
 static gpgme_error_t
 set_recipients (engine_uiserver_t uiserver, gpgme_key_t recp[])
 {
   gpgme_error_t err = 0;
+  assuan_context_t ctx = uiserver->assuan_ctx;
   char *line;
   int linelen;
   int invalid_recipients = 0;
@@ -1027,7 +1016,7 @@ set_recipients (engine_uiserver_t uiserver, gpgme_key_t recp[])
   linelen = 10 + 40 + 1;	/* "RECIPIENT " + guess + '\0'.  */
   line = malloc (10 + 40 + 1);
   if (!line)
-    return gpg_error_from_syserror ();
+    return gpg_error_from_errno (errno);
   strcpy (line, "RECIPIENT ");
   for (i=0; !err && recp[i]; i++)
     {
@@ -1047,9 +1036,9 @@ set_recipients (engine_uiserver_t uiserver, gpgme_key_t recp[])
 	  char *newline = realloc (line, newlen);
 	  if (! newline)
 	    {
-	      int saved_err = gpg_error_from_syserror ();
+	      int saved_errno = errno;
 	      free (line);
-	      return saved_err;
+	      return gpg_error_from_errno (saved_errno);
 	    }
 	  line = newline;
 	  linelen = newlen;
@@ -1057,8 +1046,7 @@ set_recipients (engine_uiserver_t uiserver, gpgme_key_t recp[])
       /* FIXME: need to do proper escaping  */
       strcpy (&line[10], uid);
 
-      err = uiserver_assuan_simple_command (uiserver, line,
-                                            uiserver->status.fnc,
+      err = uiserver_assuan_simple_command (ctx, line, uiserver->status.fnc,
                                             uiserver->status.fnc_value);
       /* FIXME: This might requires more work.  */
       if (gpg_err_code (err) == GPG_ERR_NO_PUBKEY)
@@ -1075,58 +1063,8 @@ set_recipients (engine_uiserver_t uiserver, gpgme_key_t recp[])
 }
 
 
-/* Take recipients from the LF delimited STRING and send RECIPIENT
- * commands to gpgsm.  */
 static gpgme_error_t
-set_recipients_from_string (engine_uiserver_t uiserver, const char *string)
-{
-  gpg_error_t err = 0;
-  char *line = NULL;
-  int no_pubkey = 0;
-  const char *s;
-  int n;
-
-  for (;;)
-    {
-      while (*string == ' ' || *string == '\t')
-        string++;
-      if (!*string)
-        break;
-
-      s = strchr (string, '\n');
-      if (s)
-        n = s - string;
-      else
-        n = strlen (string);
-      while (n && (string[n-1] == ' ' || string[n-1] == '\t'))
-        n--;
-
-      gpgrt_free (line);
-      if (gpgrt_asprintf (&line, "RECIPIENT %.*s", n, string) < 0)
-        {
-          err = gpg_error_from_syserror ();
-          break;
-        }
-      string += n + !!s;
-
-      err = uiserver_assuan_simple_command (uiserver, line,
-                                            uiserver->status.fnc,
-                                            uiserver->status.fnc_value);
-
-      /* Fixme: Improve error reporting.  */
-      if (gpg_err_code (err) == GPG_ERR_NO_PUBKEY)
-	no_pubkey++;
-      else if (err)
-        break;
-    }
-  gpgrt_free (line);
-  return err? err : no_pubkey? gpg_error (GPG_ERR_NO_PUBKEY) : 0;
-}
-
-
-static gpgme_error_t
-uiserver_encrypt (void *engine, gpgme_key_t recp[], const char *recpstring,
-                  gpgme_encrypt_flags_t flags,
+uiserver_encrypt (void *engine, gpgme_key_t recp[], gpgme_encrypt_flags_t flags,
 		  gpgme_data_t plain, gpgme_data_t ciph, int use_armor)
 {
   engine_uiserver_t uiserver = engine;
@@ -1150,18 +1088,18 @@ uiserver_encrypt (void *engine, gpgme_key_t recp[], const char *recpstring,
       if (!recp || plain || ciph)
 	return gpg_error (GPG_ERR_INV_VALUE);
 
-      if (gpgrt_asprintf (&cmd, "PREP_ENCRYPT%s%s", protocol,
+      if (asprintf (&cmd, "PREP_ENCRYPT%s%s", protocol,
 		    (flags & GPGME_ENCRYPT_EXPECT_SIGN)
 		    ? " --expect-sign" : "") < 0)
-	return gpg_error_from_syserror ();
+	return gpg_error_from_errno (errno);
     }
   else
     {
       if (!plain || !ciph)
 	return gpg_error (GPG_ERR_INV_VALUE);
 
-      if (gpgrt_asprintf (&cmd, "ENCRYPT%s", protocol) < 0)
-	return gpg_error_from_syserror ();
+      if (asprintf (&cmd, "ENCRYPT%s", protocol) < 0)
+	return gpg_error_from_errno (errno);
     }
 
   if (plain)
@@ -1171,11 +1109,11 @@ uiserver_encrypt (void *engine, gpgme_key_t recp[], const char *recpstring,
 			     map_data_enc (uiserver->input_cb.data));
       if (err)
 	{
-	  gpgrt_free (cmd);
+	  free (cmd);
 	  return err;
 	}
     }
-
+    
   if (ciph)
     {
       uiserver->output_cb.data = ciph;
@@ -1183,28 +1121,25 @@ uiserver_encrypt (void *engine, gpgme_key_t recp[], const char *recpstring,
 			     : map_data_enc (uiserver->output_cb.data));
       if (err)
 	{
-	  gpgrt_free (cmd);
+	  free (cmd);
 	  return err;
 	}
     }
 
   uiserver->inline_data = NULL;
 
-  if (recp || recpstring)
+  if (recp)
     {
-      if (recp)
-        err = set_recipients (uiserver, recp);
-      else
-        err = set_recipients_from_string (uiserver, recpstring);
+      err = set_recipients (uiserver, recp);
       if (err)
 	{
-	  gpgrt_free (cmd);
+	  free (cmd);
 	  return err;
 	}
     }
 
   err = start (uiserver, cmd);
-  gpgrt_free (cmd);
+  free (cmd);
   return err;
 }
 
@@ -1220,9 +1155,6 @@ uiserver_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
   char *cmd;
   gpgme_key_t key;
 
-  (void)use_textmode;
-  (void)include_certs;
-
   if (!uiserver || !in || !out)
     return gpg_error (GPG_ERR_INV_VALUE);
   if (uiserver->protocol == GPGME_PROTOCOL_DEFAULT)
@@ -1234,35 +1166,35 @@ uiserver_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
   else
     return gpgme_error (GPG_ERR_UNSUPPORTED_PROTOCOL);
 
-  if (gpgrt_asprintf (&cmd, "SIGN%s%s", protocol,
+  if (asprintf (&cmd, "SIGN%s%s", protocol,
 		(mode == GPGME_SIG_MODE_DETACH) ? " --detached" : "") < 0)
-    return gpg_error_from_syserror ();
+    return gpg_error_from_errno (errno);
 
   key = gpgme_signers_enum (ctx, 0);
   if (key)
     {
       const char *s = NULL;
-
+      
       if (key && key->uids)
         s = key->uids->email;
-
+      
       if (s && strlen (s) < 80)
         {
           char buf[100];
-
+          
           strcpy (stpcpy (buf, "SENDER --info "), s);
-          err = uiserver_assuan_simple_command (uiserver, buf,
+          err = uiserver_assuan_simple_command (uiserver->assuan_ctx, buf,
                                                 uiserver->status.fnc,
                                                 uiserver->status.fnc_value);
         }
       else
         err = gpg_error (GPG_ERR_INV_VALUE);
       gpgme_key_unref (key);
-      if (err)
-        {
-          gpgrt_free (cmd);
-          return err;
-        }
+      if (err) 
+      {
+	free (cmd);
+	return err;
+      }
   }
 
   uiserver->input_cb.data = in;
@@ -1270,7 +1202,7 @@ uiserver_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
 			 map_data_enc (uiserver->input_cb.data));
   if (err)
     {
-      gpgrt_free (cmd);
+      free (cmd);
       return err;
     }
   uiserver->output_cb.data = out;
@@ -1278,13 +1210,13 @@ uiserver_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
 			 : map_data_enc (uiserver->output_cb.data));
   if (err)
     {
-      gpgrt_free (cmd);
+      free (cmd);
       return err;
     }
   uiserver->inline_data = NULL;
 
   err = start (uiserver, cmd);
-  gpgrt_free (cmd);
+  free (cmd);
   return err;
 }
 
@@ -1292,16 +1224,13 @@ uiserver_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
 /* FIXME: Missing a way to specify --silent.  */
 static gpgme_error_t
 uiserver_verify (void *engine, gpgme_data_t sig, gpgme_data_t signed_text,
-                 gpgme_data_t plaintext, gpgme_ctx_t ctx)
+	      gpgme_data_t plaintext)
 {
   engine_uiserver_t uiserver = engine;
   gpgme_error_t err;
   const char *protocol;
   char *cmd;
 
-  (void)ctx; /* FIXME: We should to add a --sender option to the
-              * UISever protocol.  */
-
   if (!uiserver)
     return gpg_error (GPG_ERR_INV_VALUE);
   if (uiserver->protocol == GPGME_PROTOCOL_DEFAULT)
@@ -1313,15 +1242,15 @@ uiserver_verify (void *engine, gpgme_data_t sig, gpgme_data_t signed_text,
   else
     return gpgme_error (GPG_ERR_UNSUPPORTED_PROTOCOL);
 
-  if (gpgrt_asprintf (&cmd, "VERIFY%s", protocol) < 0)
-    return gpg_error_from_syserror ();
+  if (asprintf (&cmd, "VERIFY%s", protocol) < 0)
+    return gpg_error_from_errno (errno);
 
   uiserver->input_cb.data = sig;
   err = uiserver_set_fd (uiserver, INPUT_FD,
 			 map_data_enc (uiserver->input_cb.data));
   if (err)
     {
-      gpgrt_free (cmd);
+      free (cmd);
       return err;
     }
   if (plaintext)
@@ -1341,26 +1270,14 @@ uiserver_verify (void *engine, gpgme_data_t sig, gpgme_data_t signed_text,
   if (!err)
     err = start (uiserver, cmd);
 
-  gpgrt_free (cmd);
+  free (cmd);
   return err;
 }
 
 
-/* This sets a status callback for monitoring status lines before they
- * are passed to a caller set handler.  */
-static void
-uiserver_set_status_cb (void *engine, gpgme_status_cb_t cb, void *cb_value)
-{
-  engine_uiserver_t uiserver = engine;
-
-  uiserver->status.mon_cb = cb;
-  uiserver->status.mon_cb_value = cb_value;
-}
-
-
 static void
 uiserver_set_status_handler (void *engine, engine_status_handler_t fnc,
-			  void *fnc_value)
+			  void *fnc_value) 
 {
   engine_uiserver_t uiserver = engine;
 
@@ -1371,7 +1288,7 @@ uiserver_set_status_handler (void *engine, engine_status_handler_t fnc,
 
 static gpgme_error_t
 uiserver_set_colon_line_handler (void *engine, engine_colon_line_handler_t fnc,
-			      void *fnc_value)
+			      void *fnc_value) 
 {
   engine_uiserver_t uiserver = engine;
 
@@ -1395,7 +1312,7 @@ uiserver_io_event (void *engine, gpgme_event_io_t type, void *type_data)
 {
   engine_uiserver_t uiserver = engine;
 
-  TRACE (DEBUG_ENGINE, "gpgme:uiserver_io_event", uiserver,
+  TRACE3 (DEBUG_ENGINE, "gpgme:uiserver_io_event", uiserver,
           "event %p, type %d, type_data %p",
           uiserver->io_cbs.event, type, type_data);
   if (uiserver->io_cbs.event)
@@ -1406,7 +1323,7 @@ uiserver_io_event (void *engine, gpgme_event_io_t type, void *type_data)
 struct engine_ops _gpgme_engine_ops_uiserver =
   {
     /* Static functions.  */
-    _gpgme_get_default_uisrv_socket,
+    _gpgme_get_uiserver_socket_path,
     NULL,
     uiserver_get_version,
     uiserver_get_req_version,
@@ -1415,14 +1332,13 @@ struct engine_ops _gpgme_engine_ops_uiserver =
     /* Member functions.  */
     uiserver_release,
     uiserver_reset,
-    uiserver_set_status_cb,
     uiserver_set_status_handler,
     NULL,		/* set_command_handler */
     uiserver_set_colon_line_handler,
     uiserver_set_locale,
     uiserver_set_protocol,
-    NULL,               /* set_engine_flags */
     uiserver_decrypt,
+    uiserver_decrypt_verify,
     NULL,		/* delete */
     NULL,		/* edit */
     uiserver_encrypt,
@@ -1433,24 +1349,15 @@ struct engine_ops _gpgme_engine_ops_uiserver =
     NULL,		/* import */
     NULL,		/* keylist */
     NULL,		/* keylist_ext */
-    NULL,               /* keylist_data */
-    NULL,               /* keysign */
-    NULL,               /* revsig */
-    NULL,               /* tofu_policy */
     uiserver_sign,
+    NULL,		/* trustlist */
     uiserver_verify,
     NULL,		/* getauditlog */
-    NULL,               /* setexpire */
     NULL,               /* opassuan_transact */
     NULL,		/* conf_load */
     NULL,		/* conf_save */
-    NULL,		/* conf_dir */
-    NULL,               /* query_swdb */
     uiserver_set_io_cbs,
     uiserver_io_event,
     uiserver_cancel,
-    NULL,		/* cancel_op */
-    NULL,               /* passwd */
-    NULL,               /* set_pinentry_mode */
-    NULL                /* opspawn */
+    NULL		/* cancel_op */
   };
