@@ -1,23 +1,22 @@
 /* engine.c - GPGME engine support.
- * Copyright (C) 2000 Werner Koch (dd9jn)
- * Copyright (C) 2001, 2002, 2003, 2004, 2006, 2009, 2010 g10 Code GmbH
- *
- * This file is part of GPGME.
- *
- * GPGME is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * GPGME is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
- */
+   Copyright (C) 2000 Werner Koch (dd9jn)
+   Copyright (C) 2001, 2002, 2003, 2004, 2006, 2009, 2010 g10 Code GmbH
+ 
+   This file is part of GPGME.
+ 
+   GPGME is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
+   GPGME is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, see <http://www.gnu.org/licenses/>.
+*/
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -31,7 +30,6 @@
 #include "util.h"
 #include "sema.h"
 #include "ops.h"
-#include "debug.h"
 
 #include "engine.h"
 #include "engine-backend.h"
@@ -47,16 +45,31 @@ struct engine
 static struct engine_ops *engine_ops[] =
   {
     &_gpgme_engine_ops_gpg,		/* OpenPGP.  */
+#ifdef ENABLE_GPGSM
     &_gpgme_engine_ops_gpgsm,		/* CMS.  */
-    &_gpgme_engine_ops_gpgconf,		/* gpg-conf.  */
-    &_gpgme_engine_ops_assuan,		/* Low-Level Assuan.  */
-    &_gpgme_engine_ops_g13,		/* Crypto VFS.  */
-#ifdef ENABLE_UISERVER
-    &_gpgme_engine_ops_uiserver,	/* UI-Server.  */
 #else
     NULL,
 #endif
-    &_gpgme_engine_ops_spawn
+#ifdef ENABLE_GPGCONF
+    &_gpgme_engine_ops_gpgconf,		/* gpg-conf.  */
+#else
+    NULL,
+#endif
+#ifdef ENABLE_ASSUAN
+    &_gpgme_engine_ops_assuan,		/* Low-Level Assuan.  */
+#else
+    NULL,
+#endif
+#ifdef ENABLE_G13
+    &_gpgme_engine_ops_g13,		/* Crypto VFS.  */
+#else
+    NULL,
+#endif
+#ifdef ENABLE_UISERVER
+    &_gpgme_engine_ops_uiserver		/* Crypto VFS.  */
+#else
+    NULL
+#endif
   };
 
 
@@ -64,10 +77,6 @@ static struct engine_ops *engine_ops[] =
 static gpgme_engine_info_t engine_info;
 DEFINE_STATIC_LOCK (engine_info_lock);
 
-/* If non-NULL, the minimal version required for all engines.  */
-static char *engine_minimal_version;
-
-
 
 /* Get the file name of the engine for PROTOCOL.  */
 static const char *
@@ -98,8 +107,7 @@ engine_get_home_dir (gpgme_protocol_t proto)
 
 
 /* Get a malloced string containing the version number of the engine
- * for PROTOCOL.  If this function returns NULL for a valid protocol,
- * it should be assumed that the engine is a pseudo engine. */
+   for PROTOCOL.  */
 static char *
 engine_get_version (gpgme_protocol_t proto, const char *file_name)
 {
@@ -113,8 +121,7 @@ engine_get_version (gpgme_protocol_t proto, const char *file_name)
 }
 
 
-/* Get the required version number of the engine for PROTOCOL.  This
- * may be NULL. */
+/* Get the required version number of the engine for PROTOCOL.  */
 static const char *
 engine_get_req_version (gpgme_protocol_t proto)
 {
@@ -159,7 +166,7 @@ gpgme_engine_check_version (gpgme_protocol_t proto)
 				      info->req_version);
 
   UNLOCK (engine_info_lock);
-  return result ? 0 : trace_gpg_error (GPG_ERR_INV_ENGINE);
+  return result ? 0 : gpg_error (GPG_ERR_INV_ENGINE);
 }
 
 
@@ -171,8 +178,8 @@ _gpgme_engine_info_release (gpgme_engine_info_t info)
     {
       gpgme_engine_info_t next_info = info->next;
 
-      if (info->file_name)
-        free (info->file_name);
+      assert (info->file_name);
+      free (info->file_name);
       if (info->home_dir)
 	free (info->home_dir);
       if (info->version)
@@ -183,26 +190,6 @@ _gpgme_engine_info_release (gpgme_engine_info_t info)
 }
 
 
-/* This is an internal function to set a mimimal required version.
- * This function must only be called by gpgme_set_global_flag.
- * Returns 0 on success.  */
-int
-_gpgme_set_engine_minimal_version (const char *value)
-{
-  free (engine_minimal_version);
-  if (value)
-    {
-      engine_minimal_version = strdup (value);
-      return !engine_minimal_version;
-    }
-  else
-    {
-      engine_minimal_version = NULL;
-      return 0;
-    }
-}
-
-
 /* Get the information about the configured and installed engines.  A
    pointer to the first engine in the statically allocated linked list
    is returned in *INFO.  If an error occurs, it is returned.  The
@@ -210,8 +197,6 @@ _gpgme_set_engine_minimal_version (const char *value)
 gpgme_error_t
 gpgme_get_engine_info (gpgme_engine_info_t *info)
 {
-  gpgme_error_t err;
-
   LOCK (engine_info_lock);
   if (!engine_info)
     {
@@ -221,16 +206,13 @@ gpgme_get_engine_info (gpgme_engine_info_t *info)
 					GPGME_PROTOCOL_GPGCONF,
 					GPGME_PROTOCOL_ASSUAN,
 					GPGME_PROTOCOL_G13,
-					GPGME_PROTOCOL_UISERVER,
-                                        GPGME_PROTOCOL_SPAWN    };
+					GPGME_PROTOCOL_UISERVER };
       unsigned int proto;
 
-      err = 0;
       for (proto = 0; proto < DIM (proto_list); proto++)
 	{
 	  const char *ofile_name = engine_get_file_name (proto_list[proto]);
 	  const char *ohome_dir  = engine_get_home_dir (proto_list[proto]);
-          char *version = engine_get_version (proto_list[proto], NULL);
 	  char *file_name;
 	  char *home_dir;
 
@@ -238,39 +220,13 @@ gpgme_get_engine_info (gpgme_engine_info_t *info)
 	    continue;
 
 	  file_name = strdup (ofile_name);
-          if (!file_name)
-            err = gpg_error_from_syserror ();
+          home_dir = ohome_dir? strdup (ohome_dir): NULL;
 
-          if (ohome_dir)
-            {
-              home_dir = strdup (ohome_dir);
-              if (!home_dir && !err)
-                err = gpg_error_from_syserror ();
-            }
-          else
-            home_dir = NULL;
-
-	  *lastp = calloc (1, sizeof (*engine_info));
-          if (!*lastp && !err)
-            err = gpg_error_from_syserror ();
-
-          /* Check against the optional minimal engine version.  */
-          if (!err && version && engine_minimal_version
-              && !_gpgme_compare_versions (version, engine_minimal_version))
-            {
-              err = gpg_error (GPG_ERR_ENGINE_TOO_OLD);
-            }
-
-          /* Now set the dummy version for pseudo engines.  */
-          if (!err && !version)
-            {
-              version = strdup ("1.0.0");
-              if (!version)
-                err = gpg_error_from_syserror ();
-            }
-
-	  if (err)
+	  *lastp = malloc (sizeof (*engine_info));
+	  if (!*lastp || !file_name)
 	    {
+	      int saved_errno = errno;
+
 	      _gpgme_engine_info_release (engine_info);
 	      engine_info = NULL;
 
@@ -278,20 +234,16 @@ gpgme_get_engine_info (gpgme_engine_info_t *info)
 		free (file_name);
 	      if (home_dir)
 		free (home_dir);
-	      if (version)
-		free (version);
 
 	      UNLOCK (engine_info_lock);
-	      return err;
+	      return gpg_error_from_errno (saved_errno);
 	    }
 
 	  (*lastp)->protocol = proto_list[proto];
 	  (*lastp)->file_name = file_name;
 	  (*lastp)->home_dir = home_dir;
-	  (*lastp)->version = version;
+	  (*lastp)->version = engine_get_version (proto_list[proto], NULL);
 	  (*lastp)->req_version = engine_get_req_version (proto_list[proto]);
-	  if (!(*lastp)->req_version)
-            (*lastp)->req_version = "1.0.0"; /* Dummy for pseudo engines. */
 	  (*lastp)->next = NULL;
 	  lastp = &(*lastp)->next;
 	}
@@ -336,14 +288,12 @@ _gpgme_engine_info_copy (gpgme_engine_info_t *r_info)
 
       assert (info->file_name);
       file_name = strdup (info->file_name);
-      if (!file_name)
-        err = gpg_error_from_syserror ();
 
       if (info->home_dir)
 	{
 	  home_dir = strdup (info->home_dir);
-	  if (!home_dir && !err)
-	    err = gpg_error_from_syserror ();
+	  if (!home_dir)
+	    err = gpg_error_from_errno (errno);
 	}
       else
 	home_dir = NULL;
@@ -351,19 +301,19 @@ _gpgme_engine_info_copy (gpgme_engine_info_t *r_info)
       if (info->version)
 	{
 	  version = strdup (info->version);
-	  if (!version && !err)
-	    err = gpg_error_from_syserror ();
+	  if (!version)
+	    err = gpg_error_from_errno (errno);
 	}
       else
 	version = NULL;
 
       *lastp = malloc (sizeof (*engine_info));
-      if (!*lastp && !err)
-        err = gpg_error_from_syserror ();
-
-      if (err)
+      if (!*lastp || !file_name || err)
 	{
+	  int saved_errno = errno;
+
 	  _gpgme_engine_info_release (new_info);
+
 	  if (file_name)
 	    free (file_name);
 	  if (home_dir)
@@ -372,7 +322,7 @@ _gpgme_engine_info_copy (gpgme_engine_info_t *r_info)
 	    free (version);
 
 	  UNLOCK (engine_info_lock);
-	  return err;
+	  return gpg_error_from_errno (saved_errno);
 	}
 
       (*lastp)->protocol = info->protocol;
@@ -400,7 +350,6 @@ _gpgme_set_engine_info (gpgme_engine_info_t info, gpgme_protocol_t proto,
 {
   char *new_file_name;
   char *new_home_dir;
-  char *new_version;
 
   /* FIXME: Use some PROTO_MAX definition.  */
   if (proto > DIM (engine_ops))
@@ -410,7 +359,7 @@ _gpgme_set_engine_info (gpgme_engine_info_t info, gpgme_protocol_t proto,
     info = info->next;
 
   if (!info)
-    return trace_gpg_error (GPG_ERR_INV_ENGINE);
+    return gpg_error (GPG_ERR_INV_ENGINE);
 
   /* Prepare new members.  */
   if (file_name)
@@ -422,7 +371,7 @@ _gpgme_set_engine_info (gpgme_engine_info_t info, gpgme_protocol_t proto,
       new_file_name = strdup (ofile_name);
     }
   if (!new_file_name)
-    return gpg_error_from_syserror ();
+    return gpg_error_from_errno (errno);
 
   if (home_dir)
     {
@@ -430,7 +379,7 @@ _gpgme_set_engine_info (gpgme_engine_info_t info, gpgme_protocol_t proto,
       if (!new_home_dir)
 	{
 	  free (new_file_name);
-	  return gpg_error_from_syserror ();
+	  return gpg_error_from_errno (errno);
 	}
     }
   else
@@ -442,25 +391,13 @@ _gpgme_set_engine_info (gpgme_engine_info_t info, gpgme_protocol_t proto,
           if (!new_home_dir)
             {
               free (new_file_name);
-              return gpg_error_from_syserror ();
+              return gpg_error_from_errno (errno);
             }
         }
       else
         new_home_dir = NULL;
     }
 
-  new_version = engine_get_version (proto, new_file_name);
-  if (!new_version)
-    {
-      new_version = strdup ("1.0.0"); /* Fake one for dummy entries.  */
-      if (!new_version)
-        {
-          free (new_file_name);
-          free (new_home_dir);
-          return gpg_error_from_syserror ();
-        }
-    }
-
   /* Remove the old members.  */
   assert (info->file_name);
   free (info->file_name);
@@ -472,7 +409,7 @@ _gpgme_set_engine_info (gpgme_engine_info_t info, gpgme_protocol_t proto,
   /* Install the new members.  */
   info->file_name = new_file_name;
   info->home_dir = new_home_dir;
-  info->version = new_version;
+  info->version = engine_get_version (proto, new_file_name);
 
   return 0;
 }
@@ -512,19 +449,18 @@ _gpgme_engine_new (gpgme_engine_info_t info, engine_t *r_engine)
   engine_t engine;
 
   if (!info->file_name || !info->version)
-    return trace_gpg_error (GPG_ERR_INV_ENGINE);
+    return gpg_error (GPG_ERR_INV_ENGINE);
 
   engine = calloc (1, sizeof *engine);
   if (!engine)
-    return gpg_error_from_syserror ();
+    return gpg_error_from_errno (errno);
 
   engine->ops = engine_ops[info->protocol];
   if (engine->ops->new)
     {
       gpgme_error_t err;
       err = (*engine->ops->new) (&engine->engine,
-				 info->file_name, info->home_dir,
-                                 info->version);
+				 info->file_name, info->home_dir);
       if (err)
 	{
 	  free (engine);
@@ -564,21 +500,6 @@ _gpgme_engine_release (engine_t engine)
 }
 
 
-/* Set a status callback which is used to monitor the status values
- * before they are passed to a handler set with
- * _gpgme_engine_set_status_handler.  */
-void
-_gpgme_engine_set_status_cb (engine_t engine,
-                             gpgme_status_cb_t cb, void *cb_value)
-{
-  if (!engine)
-    return;
-
-  if (engine->ops->set_status_cb)
-    (*engine->ops->set_status_cb) (engine->engine, cb, cb_value);
-}
-
-
 void
 _gpgme_engine_set_status_handler (engine_t engine,
 				  engine_status_handler_t fnc, void *fnc_value)
@@ -594,7 +515,8 @@ _gpgme_engine_set_status_handler (engine_t engine,
 gpgme_error_t
 _gpgme_engine_set_command_handler (engine_t engine,
 				   engine_command_handler_t fnc,
-				   void *fnc_value)
+				   void *fnc_value,
+				   gpgme_data_t linked_data)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -602,7 +524,8 @@ _gpgme_engine_set_command_handler (engine_t engine,
   if (!engine->ops->set_command_handler)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->set_command_handler) (engine->engine, fnc, fnc_value);
+  return (*engine->ops->set_command_handler) (engine->engine,
+					      fnc, fnc_value, linked_data);
 }
 
 gpgme_error_t
@@ -645,35 +568,11 @@ _gpgme_engine_set_protocol (engine_t engine, gpgme_protocol_t protocol)
 
   return (*engine->ops->set_protocol) (engine->engine, protocol);
 }
-
-
-/* Pass information about the current context to the engine.  The
- * engine may use this context to retrieve context specific flags.
- * Important: The engine is required to immediately copy the required
- * flags to its own context!
- *
- * This function will eventually be used to reduce the number of
- * explicit passed flags.  */
-void
-_gpgme_engine_set_engine_flags (engine_t engine, gpgme_ctx_t ctx)
-{
-  if (!engine)
-    return;
-
-  if (!engine->ops->set_engine_flags)
-    return;
-
-  (*engine->ops->set_engine_flags) (engine->engine, ctx);
-}
-
+  
 
 gpgme_error_t
-_gpgme_engine_op_decrypt (engine_t engine,
-                          gpgme_decrypt_flags_t flags,
-                          gpgme_data_t ciph,
-			  gpgme_data_t plain, int export_session_key,
-                          const char *override_session_key,
-                          int auto_key_retrieve)
+_gpgme_engine_op_decrypt (engine_t engine, gpgme_data_t ciph,
+			  gpgme_data_t plain)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -681,15 +580,27 @@ _gpgme_engine_op_decrypt (engine_t engine,
   if (!engine->ops->decrypt)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->decrypt) (engine->engine, flags, ciph, plain,
-                                  export_session_key, override_session_key,
-                                  auto_key_retrieve);
+  return (*engine->ops->decrypt) (engine->engine, ciph, plain);
+}
+
+
+gpgme_error_t
+_gpgme_engine_op_decrypt_verify (engine_t engine, gpgme_data_t ciph,
+				 gpgme_data_t plain)
+{
+  if (!engine)
+    return gpg_error (GPG_ERR_INV_VALUE);
+
+  if (!engine->ops->decrypt_verify)
+    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+
+  return (*engine->ops->decrypt_verify) (engine->engine, ciph, plain);
 }
 
 
 gpgme_error_t
 _gpgme_engine_op_delete (engine_t engine, gpgme_key_t key,
-			 unsigned int flags)
+			 int allow_secret)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -697,7 +608,7 @@ _gpgme_engine_op_delete (engine_t engine, gpgme_key_t key,
   if (!engine->ops->delete)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->delete) (engine->engine, key, flags);
+  return (*engine->ops->delete) (engine->engine, key, allow_secret);
 }
 
 
@@ -717,7 +628,6 @@ _gpgme_engine_op_edit (engine_t engine, int type, gpgme_key_t key,
 
 gpgme_error_t
 _gpgme_engine_op_encrypt (engine_t engine, gpgme_key_t recp[],
-                          const char *recpstring,
 			  gpgme_encrypt_flags_t flags,
 			  gpgme_data_t plain, gpgme_data_t ciph, int use_armor)
 {
@@ -727,14 +637,13 @@ _gpgme_engine_op_encrypt (engine_t engine, gpgme_key_t recp[],
   if (!engine->ops->encrypt)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->encrypt) (engine->engine, recp, recpstring,
-                                  flags, plain, ciph, use_armor);
+  return (*engine->ops->encrypt) (engine->engine, recp, flags, plain, ciph,
+				  use_armor);
 }
 
 
 gpgme_error_t
 _gpgme_engine_op_encrypt_sign (engine_t engine, gpgme_key_t recp[],
-                               const char *recpstring,
 			       gpgme_encrypt_flags_t flags,
 			       gpgme_data_t plain, gpgme_data_t ciph,
 			       int use_armor, gpgme_ctx_t ctx /* FIXME */)
@@ -745,8 +654,8 @@ _gpgme_engine_op_encrypt_sign (engine_t engine, gpgme_key_t recp[],
   if (!engine->ops->encrypt_sign)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->encrypt_sign) (engine->engine, recp, recpstring,
-                                       flags, plain, ciph, use_armor, ctx);
+  return (*engine->ops->encrypt_sign) (engine->engine, recp, flags,
+				       plain, ciph, use_armor, ctx);
 }
 
 
@@ -783,13 +692,9 @@ _gpgme_engine_op_export_ext (engine_t engine, const char *pattern[],
 
 
 gpgme_error_t
-_gpgme_engine_op_genkey (engine_t engine,
-                         const char *userid, const char *algo,
-                         unsigned long reserved, unsigned long expires,
-                         gpgme_key_t key, unsigned int flags,
-                         gpgme_data_t help_data,
-			 unsigned int extraflags,
-                         gpgme_data_t pubkey, gpgme_data_t seckey)
+_gpgme_engine_op_genkey (engine_t engine, gpgme_data_t help_data,
+			 int use_armor, gpgme_data_t pubkey,
+			 gpgme_data_t seckey)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -797,61 +702,14 @@ _gpgme_engine_op_genkey (engine_t engine,
   if (!engine->ops->genkey)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->genkey) (engine->engine,
-                                 userid, algo, reserved, expires, key, flags,
-                                 help_data, extraflags,
+  return (*engine->ops->genkey) (engine->engine, help_data, use_armor,
 				 pubkey, seckey);
 }
 
 
-gpgme_error_t
-_gpgme_engine_op_keysign (engine_t engine, gpgme_key_t key, const char *userid,
-                          unsigned long expires, unsigned int flags,
-                          gpgme_ctx_t ctx)
-{
-  if (!engine)
-    return gpg_error (GPG_ERR_INV_VALUE);
-
-  if (!engine->ops->keysign)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-
-  return (*engine->ops->keysign) (engine->engine,
-                                  key, userid, expires, flags, ctx);
-}
-
-
-gpgme_error_t
-_gpgme_engine_op_revsig (engine_t engine, gpgme_key_t key, gpgme_key_t signing_key,
-                         const char *userid, unsigned int flags)
-{
-  if (!engine)
-    return gpg_error (GPG_ERR_INV_VALUE);
-
-  if (!engine->ops->revsig)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-
-  return (*engine->ops->revsig) (engine->engine, key, signing_key, userid, flags);
-}
-
-
-gpgme_error_t
-_gpgme_engine_op_tofu_policy (engine_t engine,
-                              gpgme_key_t key,  gpgme_tofu_policy_t policy)
-{
-  if (!engine)
-    return gpg_error (GPG_ERR_INV_VALUE);
-
-  if (!engine->ops->tofu_policy)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-
-  return (*engine->ops->tofu_policy) (engine->engine, key, policy);
-}
-
-
 gpgme_error_t
 _gpgme_engine_op_import (engine_t engine, gpgme_data_t keydata,
-                         gpgme_key_t *keyarray, const char *keyids[],
-                         const char *import_filter, const char *key_origin)
+                         gpgme_key_t *keyarray)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -859,15 +717,13 @@ _gpgme_engine_op_import (engine_t engine, gpgme_data_t keydata,
   if (!engine->ops->import)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->import) (engine->engine, keydata, keyarray, keyids,
-                                 import_filter, key_origin);
+  return (*engine->ops->import) (engine->engine, keydata, keyarray);
 }
 
 
 gpgme_error_t
 _gpgme_engine_op_keylist (engine_t engine, const char *pattern,
-			  int secret_only, gpgme_keylist_mode_t mode,
-			  int engine_flags)
+			  int secret_only, gpgme_keylist_mode_t mode)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -875,15 +731,14 @@ _gpgme_engine_op_keylist (engine_t engine, const char *pattern,
   if (!engine->ops->keylist)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->keylist) (engine->engine, pattern, secret_only, mode,
-                                  engine_flags);
+  return (*engine->ops->keylist) (engine->engine, pattern, secret_only, mode);
 }
 
 
 gpgme_error_t
 _gpgme_engine_op_keylist_ext (engine_t engine, const char *pattern[],
 			      int secret_only, int reserved,
-			      gpgme_keylist_mode_t mode, int engine_flags)
+			      gpgme_keylist_mode_t mode)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -892,21 +747,7 @@ _gpgme_engine_op_keylist_ext (engine_t engine, const char *pattern[],
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
   return (*engine->ops->keylist_ext) (engine->engine, pattern, secret_only,
-				      reserved, mode, engine_flags);
-}
-
-
-gpgme_error_t
-_gpgme_engine_op_keylist_data (engine_t engine, gpgme_keylist_mode_t mode,
-			       gpgme_data_t data)
-{
-  if (!engine)
-    return gpg_error (GPG_ERR_INV_VALUE);
-
-  if (!engine->ops->keylist_data)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-
-  return (*engine->ops->keylist_data) (engine->engine, mode, data);
+				      reserved, mode);
 }
 
 
@@ -930,19 +771,19 @@ _gpgme_engine_op_sign (engine_t engine, gpgme_data_t in, gpgme_data_t out,
 gpgme_error_t
 _gpgme_engine_op_trustlist (engine_t engine, const char *pattern)
 {
-  (void)pattern;
-
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
 
-  return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+  if (!engine->ops->trustlist)
+    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+
+  return (*engine->ops->trustlist) (engine->engine, pattern);
 }
 
 
 gpgme_error_t
 _gpgme_engine_op_verify (engine_t engine, gpgme_data_t sig,
-			 gpgme_data_t signed_text, gpgme_data_t plaintext,
-                         gpgme_ctx_t ctx)
+			 gpgme_data_t signed_text, gpgme_data_t plaintext)
 {
   if (!engine)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -950,8 +791,7 @@ _gpgme_engine_op_verify (engine_t engine, gpgme_data_t sig,
   if (!engine->ops->verify)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->verify) (engine->engine, sig, signed_text, plaintext,
-                                 ctx);
+  return (*engine->ops->verify) (engine->engine, sig, signed_text, plaintext);
 }
 
 
@@ -970,7 +810,7 @@ _gpgme_engine_op_getauditlog (engine_t engine, gpgme_data_t output,
 
 
 gpgme_error_t
-_gpgme_engine_op_assuan_transact (engine_t engine,
+_gpgme_engine_op_assuan_transact (engine_t engine, 
                                   const char *command,
                                   gpgme_assuan_data_cb_t data_cb,
                                   void *data_cb_value,
@@ -985,7 +825,7 @@ _gpgme_engine_op_assuan_transact (engine_t engine,
   if (!engine->ops->opassuan_transact)
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  return (*engine->ops->opassuan_transact) (engine->engine,
+  return (*engine->ops->opassuan_transact) (engine->engine, 
                                             command,
                                             data_cb, data_cb_value,
                                             inq_cb, inq_cb_value,
@@ -1019,34 +859,6 @@ _gpgme_engine_op_conf_save (engine_t engine, gpgme_conf_comp_t conf)
 }
 
 
-gpgme_error_t
-_gpgme_engine_op_conf_dir (engine_t engine, const char *what, char **result)
-{
-  if (!engine)
-    return gpg_error (GPG_ERR_INV_VALUE);
-
-  if (!engine->ops->conf_dir)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-
-  return (*engine->ops->conf_dir) (engine->engine, what, result);
-}
-
-
-gpgme_error_t
-_gpgme_engine_op_query_swdb (engine_t engine,
-                             const char *name, const char *iversion,
-                             gpgme_query_swdb_result_t result)
-{
-  if (!engine)
-    return gpg_error (GPG_ERR_INV_VALUE);
-
-  if (!engine->ops->query_swdb)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-
-  return (*engine->ops->query_swdb) (engine->engine, name, iversion, result);
-}
-
-
 void
 _gpgme_engine_set_io_cbs (engine_t engine, gpgme_io_cbs_t io_cbs)
 {
@@ -1110,48 +922,3 @@ _gpgme_engine_op_passwd (engine_t engine, gpgme_key_t key,
   return (*engine->ops->passwd) (engine->engine, key, flags);
 }
 
-
-/* Set the pinentry mode for ENGINE to MODE.  */
-gpgme_error_t
-_gpgme_engine_set_pinentry_mode (engine_t engine, gpgme_pinentry_mode_t mode)
-{
-  if (!engine)
-    return gpg_error (GPG_ERR_INV_VALUE);
-
-  if (!engine->ops->set_pinentry_mode)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-
-  return (*engine->ops->set_pinentry_mode) (engine->engine, mode);
-}
-
-
-gpgme_error_t
-_gpgme_engine_op_spawn (engine_t engine,
-                        const char *file, const char *argv[],
-                        gpgme_data_t datain,
-                        gpgme_data_t dataout, gpgme_data_t dataerr,
-                        unsigned int flags)
-{
-  if (!engine)
-    return gpg_error (GPG_ERR_INV_VALUE);
-
-  if (!engine->ops->opspawn)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-
-  return (*engine->ops->opspawn) (engine->engine, file, argv,
-                                  datain, dataout, dataerr, flags);
-}
-
-gpgme_error_t
-_gpgme_engine_op_setexpire (engine_t engine, gpgme_key_t key,
-                            unsigned long expires, const char *subfprs,
-                            unsigned int reserved)
-{
-  if (!engine)
-    return gpg_error (GPG_ERR_INV_VALUE);
-
-  if (!engine->ops->setexpire)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-
-  return (*engine->ops->setexpire) (engine->engine, key, expires, subfprs, reserved);
-}
