@@ -1,3 +1,4 @@
+
 /* getauditlog.c - Retrieve the audit log.
    Copyright (C) 2007 g10 Code GmbH
 
@@ -22,6 +23,7 @@
 #endif
 
 #include "gpgme.h"
+#include "debug.h"
 #include "context.h"
 #include "ops.h"
 
@@ -62,7 +64,11 @@ gpgme_error_t
 gpgme_op_getauditlog_start (gpgme_ctx_t ctx, 
                             gpgme_data_t output, unsigned int flags)
 {
-  return getauditlog_start (ctx, 0, output, flags);
+  gpg_error_t err;
+  TRACE_BEG2 (DEBUG_CTX, "gpgme_op_getauditlog_start", ctx,
+	      "output=%p, flags=0x%x", output, flags);
+  err = getauditlog_start (ctx, 0, output, flags);
+  return TRACE_ERR (err);
 }
 
 
@@ -73,9 +79,14 @@ gpgme_op_getauditlog_start (gpgme_ctx_t ctx,
 gpgme_error_t
 gpgme_op_getauditlog (gpgme_ctx_t ctx, gpgme_data_t output, unsigned int flags)
 {
-  gpgme_error_t err = getauditlog_start (ctx, 1, output, flags);
+  gpgme_error_t err;
+
+  TRACE_BEG2 (DEBUG_CTX, "gpgme_op_getauditlog", ctx,
+	      "output=%p, flags=0x%x", output, flags);
+
+  err = getauditlog_start (ctx, 1, output, flags);
   if (!err)
     err = _gpgme_wait_one (ctx);
-  return err;
+  return TRACE_ERR (err);
 }
 
