@@ -130,6 +130,13 @@ _gpgme_op_reset (gpgme_ctx_t ctx, int type)
         }
     }
 
+  if (ctx->sub_protocol != GPGME_PROTOCOL_DEFAULT)
+    {
+      err = _gpgme_engine_set_protocol (ctx->engine, ctx->sub_protocol);
+      if (err)
+	return err;
+    }
+
   if (type == 1 || (type == 2 && !ctx->io_cbs.add))
     {
       /* Use private event loop.  */
@@ -162,8 +169,8 @@ _gpgme_op_reset (gpgme_ctx_t ctx, int type)
 }
 
 
-/* Parse the INV_RECP status line in ARGS and return the result in
-   KEY.  */
+/* Parse the INV_RECP or INV-SNDR status line in ARGS and return the
+   result in KEY.  */
 gpgme_error_t
 _gpgme_parse_inv_recp (char *args, gpgme_invalid_key_t *key)
 {
@@ -177,7 +184,7 @@ _gpgme_parse_inv_recp (char *args, gpgme_invalid_key_t *key)
   inv_key->next = NULL;
   errno = 0;
   reason = strtol (args, &tail, 0);
-  if (errno || args == tail || *tail != ' ')
+  if (errno || args == tail || (*tail && *tail != ' '))
     {
       /* The crypto backend does not behave.  */
       free (inv_key);
@@ -236,7 +243,7 @@ _gpgme_parse_inv_recp (char *args, gpgme_invalid_key_t *key)
       break;
     }
 
-  while (*tail == ' ')
+  while (*tail && *tail == ' ')
     tail++;
   if (*tail)
     {
