@@ -27,14 +27,19 @@
 
 
 /* From gpgme.c.  */
-gpgme_error_t _gpgme_cancel_with_err (gpgme_ctx_t ctx, gpg_error_t ctx_err);
+gpgme_error_t _gpgme_cancel_with_err (gpgme_ctx_t ctx, gpg_error_t ctx_err,
+				      gpg_error_t op_err);
+/* Clear all notation data from the context.  */
+void _gpgme_sig_notation_clear (gpgme_ctx_t ctx);
 
 void _gpgme_release_result (gpgme_ctx_t ctx);
 
 
 /* From wait.c.  */
 gpgme_error_t _gpgme_wait_one (gpgme_ctx_t ctx);
-gpgme_error_t _gpgme_wait_on_condition (gpgme_ctx_t ctx, volatile int *cond);
+gpgme_error_t _gpgme_wait_one_ext (gpgme_ctx_t ctx, gpgme_error_t *op_err);
+gpgme_error_t _gpgme_wait_on_condition (gpgme_ctx_t ctx, volatile int *cond,
+					gpgme_error_t *op_err);
 
 
 /* From data.c.  */
@@ -76,6 +81,9 @@ gpgme_error_t _gpgme_decrypt_status_handler (void *priv,
 					     char *args);
 
 
+/* From signers.c.  */
+void _gpgme_signers_clear (gpgme_ctx_t ctx);
+
 /* From sign.c.  */
 
 /* Create an initial op data object for signing.  Needs to be called
@@ -120,7 +128,7 @@ gpgme_error_t _gpgme_progress_status_handler (void *priv,
 gpgme_error_t _gpgme_key_new (gpgme_key_t *r_key);
 gpgme_error_t _gpgme_key_add_subkey (gpgme_key_t key,
 				     gpgme_subkey_t *r_subkey);
-gpgme_error_t _gpgme_key_append_name (gpgme_key_t key, char *src);
+gpgme_error_t _gpgme_key_append_name (gpgme_key_t key, char *src, int convert);
 gpgme_key_sig_t _gpgme_key_add_sig (gpgme_key_t key, char *src);
 
 
