@@ -22,6 +22,11 @@
 #ifndef IO_H
 #define IO_H
 
+#ifdef HAVE_W32_SYSTEM
+# include <windows.h>
+#else
+# include <sys/socket.h>
+#endif
 
 /* A single file descriptor passed to spawn.  For child fds, dup_to
    specifies the fd it should become in the child, but only 0, 1 and 2
@@ -50,6 +55,7 @@ struct io_select_fd_s
 
 /* These function are either defined in posix-io.c or w32-io.c.  */
 void _gpgme_io_subsystem_init (void);
+int _gpgme_io_connect (int fd, struct sockaddr *addr, int addrlen);
 int _gpgme_io_read (int fd, void *buffer, size_t count);
 int _gpgme_io_write (int fd, const void *buffer, size_t count);
 int _gpgme_io_pipe (int filedes[2], int inherit_idx);
@@ -59,12 +65,20 @@ int _gpgme_io_set_close_notify (int fd, _gpgme_close_notify_handler_t handler,
 				void *value);
 int _gpgme_io_set_nonblocking (int fd);
 
+/* A flag to tell the spawn function to allow the child process to set
+   the foreground window. */
+#define IOSPAWN_FLAG_ALLOW_SET_FG 1
+/* Don't close any child FDs.  */
+#define IOSPAWN_FLAG_NOCLOSE 2
+
 /* Spawn the executable PATH with ARGV as arguments.  After forking
    close all fds except for those in FD_LIST in the child, then
    optionally dup() the child fds.  Finally, all fds in the list are
    closed in the parent.  */
-int _gpgme_io_spawn (const char *path, char *const argv[],
-		     struct spawn_fd_item_s *fd_list, pid_t *r_pid);
+int _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
+		     struct spawn_fd_item_s *fd_list,
+		     void (*atfork) (void *opaque, int reserved),
+		     void *atforkvalue, pid_t *r_pid);
 
 int _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock);
 
@@ -82,4 +96,10 @@ int _gpgme_io_fd2str (char *buf, int buflen, int fd);
    (in engine-gpgsm.c).  */
 int _gpgme_io_dup (int fd);
 
+#ifndef HAVE_W32_SYSTEM
+int _gpgme_io_recvmsg (int fd, struct msghdr *msg, int flags);
+int _gpgme_io_sendmsg (int fd, const struct msghdr *msg, int flags);
+int _gpgme_io_waitpid (int pid, int hang, int *r_status, int *r_signal);
+#endif
+
 #endif /* IO_H */
