@@ -1,23 +1,23 @@
 /* sign.c - Signing function.
- * Copyright (C) 2000 Werner Koch (dd9jn)
- * Copyright (C) 2001, 2002, 2003, 2004, 2007 g10 Code GmbH
- *
- * This file is part of GPGME.
- *
- * GPGME is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * GPGME is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
- */
+   Copyright (C) 2000 Werner Koch (dd9jn)
+   Copyright (C) 2001, 2002, 2003, 2004, 2007 g10 Code GmbH
+
+   This file is part of GPGME.
+ 
+   GPGME is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
+   GPGME is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -39,15 +39,6 @@ typedef struct
 {
   struct _gpgme_op_sign_result result;
 
-  /* The error code from a FAILURE status line or 0.  */
-  gpg_error_t failure_code;
-
-  /* The fingerprint from the last KEY_CONSIDERED status line.  */
-  char *kc_fpr;
-
-  /* The flags from the last KEY_CONSIDERED status line.  */
-  unsigned int kc_flags;
-
   /* A pointer to the next pointer of the last invalid signer in
      the list.  This makes appending new invalid signers painless
      while preserving the order.  */
@@ -63,24 +54,12 @@ typedef struct
 } *op_data_t;
 
 
-static void
-release_signatures (gpgme_new_signature_t sig)
-{
-  while (sig)
-    {
-      gpgme_new_signature_t next = sig->next;
-      free (sig->fpr);
-      free (sig);
-      sig = next;
-    }
-}
-
-
 static void
 release_op_data (void *hook)
 {
   op_data_t opd = (op_data_t) hook;
   gpgme_invalid_key_t invalid_signer = opd->result.invalid_signers;
+  gpgme_new_signature_t sig = opd->result.signatures;
 
   while (invalid_signer)
     {
@@ -91,8 +70,13 @@ release_op_data (void *hook)
       invalid_signer = next;
     }
 
-  release_signatures (opd->result.signatures);
-  free (opd->kc_fpr);
+  while (sig)
+    {
+      gpgme_new_signature_t next = sig->next;
+      free (sig->fpr);
+      free (sig);
+      sig = next;
+    }
 }
 
 
@@ -102,107 +86,70 @@ gpgme_op_sign_result (gpgme_ctx_t ctx)
   void *hook;
   op_data_t opd;
   gpgme_error_t err;
-  gpgme_invalid_key_t inv_key, key;
-  gpgme_new_signature_t sig;
-  unsigned int inv_signers = 0;
-  unsigned int signatures = 0;
 
-  TRACE_BEG (DEBUG_CTX, "gpgme_op_sign_result", ctx, "");
+  TRACE_BEG (DEBUG_CTX, "gpgme_op_sign_result", ctx);
 
   err = _gpgme_op_data_lookup (ctx, OPDATA_SIGN, &hook, -1, NULL);
   opd = hook;
   if (err || !opd)
     {
-      TRACE_SUC ("result=(null)");
+      TRACE_SUC0 ("result=(null)");
       return NULL;
     }
 
-  for (inv_key = opd->result.invalid_signers; inv_key; inv_key = inv_key->next)
-    inv_signers++;
-  for (sig = opd->result.signatures; sig; sig = sig->next)
-    signatures++;
-
-  if (gpgme_signers_count (ctx)
-      && signatures + inv_signers != gpgme_signers_count (ctx))
+  if (_gpgme_debug_trace ())
     {
-      /* In this case at least one signatures was not created perhaps
-         due to a bad passphrase etc.  Thus the entire message is
-         broken and should not be used.  We add the already created
-         signatures to the invalid signers list and thus this case can
-         be detected.  */
-      TRACE_LOG  ("result: invalid signers: %u, signatures: %u, count: %u",
-                  inv_signers, signatures, gpgme_signers_count (ctx));
+      gpgme_invalid_key_t inv_key = opd->result.invalid_signers;
+      gpgme_new_signature_t sig = opd->result.signatures;
+      int inv_signers = 0;
+      int signatures = 0;
 
-      for (sig = opd->result.signatures; sig; sig = sig->next)
-        {
-          key = calloc (1, sizeof *key);
-          if (!key)
-            {
-              TRACE_SUC ("out of core; result=(null)");
-              return NULL;
-            }
-          if (sig->fpr)
-            {
-              key->fpr = strdup (sig->fpr);
-              if (!key->fpr)
-                {
-                  free (key);
-                  TRACE_SUC ("out of core; result=(null)");
-                  return NULL;
-                }
-            }
-          key->reason = GPG_ERR_GENERAL;
-
-          inv_key = opd->result.invalid_signers;
-          if (inv_key)
-            {
-              for (; inv_key->next; inv_key = inv_key->next)
-                ;
-              inv_key->next = key;
-            }
-          else
-            opd->result.invalid_signers = key;
-        }
-
-      release_signatures (opd->result.signatures);
-      opd->result.signatures = NULL;
-    }
-
-  if (_gpgme_debug_trace())
-    {
-      TRACE_LOG  ("result: invalid signers: %i, signatures: %i",
-		  inv_signers, signatures);
-      for (inv_key=opd->result.invalid_signers; inv_key; inv_key=inv_key->next)
+      while (inv_key)
 	{
-	  TRACE_LOG  ("result: invalid signer: fpr=%s, reason=%s <%s>",
+	  inv_signers++;
+	  inv_key = inv_key->next;
+	}
+      while (sig)
+	{
+	  signatures++;
+	  sig = sig->next;
+	}
+
+      TRACE_LOG2 ("result: invalid signers: %i, signatures: %i",
+		  inv_signers, signatures);
+      inv_key = opd->result.invalid_signers;
+      while (inv_key)
+	{
+	  TRACE_LOG3 ("result: invalid signer: fpr=%s, reason=%s <%s>",
 		      inv_key->fpr, gpgme_strerror (inv_key->reason),
 		      gpgme_strsource (inv_key->reason));
+	  inv_key = inv_key->next;
 	}
-      for (sig = opd->result.signatures; sig; sig = sig->next)
+      sig = opd->result.signatures;
+      while (sig)
 	{
-	  TRACE_LOG  ("result: signature: type=%i, pubkey_algo=%i, "
+	  TRACE_LOG6 ("result: signature: type=%i, pubkey_algo=%i, "
 		      "hash_algo=%i, timestamp=%li, fpr=%s, sig_class=%i",
 		      sig->type, sig->pubkey_algo, sig->hash_algo,
 		      sig->timestamp, sig->fpr, sig->sig_class);
+	  sig = sig->next;
 	}
-   }
+    }
 
-  TRACE_SUC ("result=%p", &opd->result);
+  TRACE_SUC1 ("result=%p", &opd->result);
   return &opd->result;
 }
 
-
 
 static gpgme_error_t
-parse_sig_created (char *args, gpgme_new_signature_t *sigp,
-                   gpgme_protocol_t protocol)
+parse_sig_created (char *args, gpgme_new_signature_t *sigp)
 {
   gpgme_new_signature_t sig;
   char *tail;
 
   sig = malloc (sizeof (*sig));
   if (!sig)
-    return gpg_error_from_syserror ();
+    return gpg_error_from_errno (errno);
 
   sig->next = NULL;
   switch (*args)
@@ -222,23 +169,23 @@ parse_sig_created (char *args, gpgme_new_signature_t *sigp,
     default:
       /* The backend engine is not behaving.  */
       free (sig);
-      return trace_gpg_error (GPG_ERR_INV_ENGINE);
+      return gpg_error (GPG_ERR_INV_ENGINE);
     }
 
   args++;
   if (*args != ' ')
     {
       free (sig);
-      return trace_gpg_error (GPG_ERR_INV_ENGINE);
+      return gpg_error (GPG_ERR_INV_ENGINE);
     }
 
-  gpg_err_set_errno (0);
-  sig->pubkey_algo = _gpgme_map_pk_algo (strtol (args, &tail, 0), protocol);
+  errno = 0;
+  sig->pubkey_algo = strtol (args, &tail, 0);
   if (errno || args == tail || *tail != ' ')
     {
       /* The crypto backend does not behave.  */
       free (sig);
-      return trace_gpg_error (GPG_ERR_INV_ENGINE);
+      return gpg_error (GPG_ERR_INV_ENGINE);
     }
   args = tail;
 
@@ -247,7 +194,7 @@ parse_sig_created (char *args, gpgme_new_signature_t *sigp,
     {
       /* The crypto backend does not behave.  */
       free (sig);
-      return trace_gpg_error (GPG_ERR_INV_ENGINE);
+      return gpg_error (GPG_ERR_INV_ENGINE);
     }
   args = tail;
 
@@ -258,7 +205,7 @@ parse_sig_created (char *args, gpgme_new_signature_t *sigp,
     {
       /* The crypto backend does not behave.  */
       free (sig);
-      return trace_gpg_error (GPG_ERR_INV_ENGINE);
+      return gpg_error (GPG_ERR_INV_ENGINE);
     }
   args = tail;
 
@@ -267,7 +214,7 @@ parse_sig_created (char *args, gpgme_new_signature_t *sigp,
     {
       /* The crypto backend does not behave.  */
       free (sig);
-      return trace_gpg_error (GPG_ERR_INV_ENGINE);
+      return gpg_error (GPG_ERR_INV_ENGINE);
     }
   args = tail;
   while (*args == ' ')
@@ -277,7 +224,7 @@ parse_sig_created (char *args, gpgme_new_signature_t *sigp,
     {
       /* The crypto backend does not behave.  */
       free (sig);
-      return trace_gpg_error (GPG_ERR_INV_ENGINE);
+      return gpg_error (GPG_ERR_INV_ENGINE);
     }
 
   tail = strchr (args, ' ');
@@ -287,8 +234,9 @@ parse_sig_created (char *args, gpgme_new_signature_t *sigp,
   sig->fpr = strdup (args);
   if (!sig->fpr)
     {
+      int saved_errno = errno;
       free (sig);
-      return gpg_error_from_syserror ();
+      return gpg_error_from_errno (saved_errno);
     }
   *sigp = sig;
   return 0;
@@ -316,45 +264,25 @@ _gpgme_sign_status_handler (void *priv, gpgme_status_code_t code, char *args)
     {
     case GPGME_STATUS_SIG_CREATED:
       opd->sig_created_seen = 1;
-      err = parse_sig_created (args, opd->last_sig_p, ctx->protocol);
+      err = parse_sig_created (args, opd->last_sig_p);
       if (err)
 	return err;
 
       opd->last_sig_p = &(*opd->last_sig_p)->next;
       break;
 
-    case GPGME_STATUS_KEY_CONSIDERED:
-      /* This is emitted during gpg's key lookup to give information
-       * about the lookup results.  We store the last one so it can be
-       * used in connection with INV_RECP.  */
-      free (opd->kc_fpr);
-      opd->kc_fpr = NULL;
-      err = _gpgme_parse_key_considered (args, &opd->kc_fpr, &opd->kc_flags);
-      if (err)
-        return err;
-      break;
-
     case GPGME_STATUS_INV_RECP:
       if (opd->inv_sgnr_seen && opd->ignore_inv_recp)
-        break;
-      /* FALLTHROUGH */
+        break; 
+      /* FALLTROUGH */
     case GPGME_STATUS_INV_SGNR:
       if (code == GPGME_STATUS_INV_SGNR)
         opd->inv_sgnr_seen = 1;
-      free (opd->kc_fpr);
-      opd->kc_fpr = NULL;
-      err = _gpgme_parse_inv_recp (args, 1, opd->kc_fpr, opd->kc_flags,
-                                   opd->last_signer_p);
+      err = _gpgme_parse_inv_recp (args, opd->last_signer_p);
       if (err)
 	return err;
 
       opd->last_signer_p = &(*opd->last_signer_p)->next;
-      free (opd->kc_fpr);
-      opd->kc_fpr = NULL;
-      break;
-
-    case GPGME_STATUS_FAILURE:
-      opd->failure_code = _gpgme_parse_failure (args);
       break;
 
     case GPGME_STATUS_EOF:
@@ -365,12 +293,7 @@ _gpgme_sign_status_handler (void *priv, gpgme_status_code_t code, char *args)
 	err = gpg_error (GPG_ERR_UNUSABLE_SECKEY);
       else if (!opd->sig_created_seen
                && ctx->protocol != GPGME_PROTOCOL_UISERVER)
-	err = opd->failure_code? opd->failure_code:gpg_error (GPG_ERR_GENERAL);
-      break;
-
-    case GPGME_STATUS_INQUIRE_MAXLEN:
-      if (ctx->status_cb && !ctx->full_status)
-        err = ctx->status_cb (ctx->status_cb_value, "INQUIRE_MAXLEN", args);
+	err = gpg_error (GPG_ERR_GENERAL);
       break;
 
     default:
@@ -404,7 +327,6 @@ sign_init_result (gpgme_ctx_t ctx, int ignore_inv_recp)
   opd = hook;
   if (err)
     return err;
-  opd->failure_code = 0;
   opd->last_signer_p = &opd->result.invalid_signers;
   opd->last_sig_p = &opd->result.signatures;
   opd->ignore_inv_recp = !!ignore_inv_recp;
@@ -449,7 +371,7 @@ sign_start (gpgme_ctx_t ctx, int synchronous, gpgme_data_t plain,
   if (ctx->passphrase_cb)
     {
       err = _gpgme_engine_set_command_handler
-	(ctx->engine, _gpgme_passphrase_command_handler, ctx);
+	(ctx->engine, _gpgme_passphrase_command_handler, ctx, NULL);
       if (err)
 	return err;
     }
@@ -469,12 +391,8 @@ gpgme_op_sign_start (gpgme_ctx_t ctx, gpgme_data_t plain, gpgme_data_t sig,
 		     gpgme_sig_mode_t mode)
 {
   gpg_error_t err;
-  TRACE_BEG  (DEBUG_CTX, "gpgme_op_sign_start", ctx,
+  TRACE_BEG3 (DEBUG_CTX, "gpgme_op_sign_start", ctx,
 	      "plain=%p, sig=%p, mode=%i", plain, sig, mode);
-
-  if (!ctx)
-    return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
-
   err = sign_start (ctx, 0, plain, sig, mode);
   return TRACE_ERR (err);
 }
@@ -487,12 +405,8 @@ gpgme_op_sign (gpgme_ctx_t ctx, gpgme_data_t plain, gpgme_data_t sig,
 {
   gpgme_error_t err;
 
-  TRACE_BEG  (DEBUG_CTX, "gpgme_op_sign", ctx,
+  TRACE_BEG3 (DEBUG_CTX, "gpgme_op_sign", ctx,
 	      "plain=%p, sig=%p, mode=%i", plain, sig, mode);
-
-  if (!ctx)
-    return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
-
   err = sign_start (ctx, 1, plain, sig, mode);
   if (!err)
     err = _gpgme_wait_one (ctx);
