@@ -2,17 +2,17 @@
    Copyright (C) 2009 g10 Code GmbH
 
    This file is part of GPGME.
-
+ 
    GPGME is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; either version 2.1 of
    the License, or (at your option) any later version.
-
+   
    GPGME is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
-
+   
    You should have received a copy of the GNU Lesser General Public
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -116,10 +116,9 @@ vfs_start (gpgme_ctx_t ctx, int synchronous,
 
 
 
-#if 0
 /* XXXX.  This is the asynchronous variant. */
 static gpgme_error_t
-gpgme_op_vfs_transact_start (gpgme_ctx_t ctx,
+gpgme_op_vfs_transact_start (gpgme_ctx_t ctx, 
 			     const char *command,
 			     gpgme_assuan_data_cb_t data_cb,
 			     void *data_cb_value,
@@ -131,7 +130,6 @@ gpgme_op_vfs_transact_start (gpgme_ctx_t ctx,
   return vfs_start (ctx, 0, command, data_cb, data_cb_value,
 		    inq_cb, inq_cb_value, status_cb, status_cb_value);
 }
-#endif
 
 
 /* XXXX.  This is the synchronous variant. */
@@ -147,10 +145,7 @@ gpgme_op_vfs_transact (gpgme_ctx_t ctx,
 		       gpgme_error_t *op_err)
 {
   gpgme_error_t err;
-
-  if (!ctx)
-    return gpg_error (GPG_ERR_INV_VALUE);
-
+  
   err = vfs_start (ctx, 1, command, data_cb, data_cb_value,
 		   inq_cb, inq_cb_value, status_cb, status_cb_value);
   if (!err)
@@ -173,8 +168,6 @@ _gpgme_op_vfs_mount (gpgme_ctx_t ctx, const char *container_file,
   char *cmd;
   char *container_file_esc = NULL;
 
-  (void)flags;
-
   /* We want to encourage people to check error values, so not getting
      them is discouraged here.  Also makes our code easier.  */
   if (! op_err)
@@ -219,7 +212,7 @@ _gpgme_op_vfs_mount (gpgme_ctx_t ctx, const char *container_file,
       if (asprintf (&cmd, "MOUNT") < 0)
 	return gpg_error_from_syserror ();
     }
-
+    
   err = gpgme_op_vfs_transact (ctx, cmd, NULL, NULL, NULL, NULL,
 			       _gpgme_vfs_mount_status_handler, ctx, op_err);
   free (cmd);
@@ -237,10 +230,6 @@ gpgme_op_vfs_mount (gpgme_ctx_t ctx, const char *container_file,
   TRACE_BEG4 (DEBUG_CTX, "gpgme_op_vfs_mount", ctx,
 	      "container=%s, mount_dir=%s, flags=0x%x, op_err=%p",
 	      container_file, mount_dir, flags, op_err);
-
-  if (!ctx)
-    return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
-
   err = _gpgme_op_vfs_mount (ctx, container_file, mount_dir, flags, op_err);
   return TRACE_ERR (err);
 }
