@@ -585,7 +585,9 @@ build_commandline (char **argv)
 
 int
 _gpgme_io_spawn (const char *path, char * const argv[], unsigned int flags,
-		 struct spawn_fd_item_s *fd_list, pid_t *r_pid)
+		 struct spawn_fd_item_s *fd_list,
+		 void (*atfork) (void *opaque, int reserved),
+		 void *atforkvalue, pid_t *r_pid)
 {
   SECURITY_ATTRIBUTES sec_attr;
   PROCESS_INFORMATION pi =
@@ -777,8 +779,11 @@ _gpgme_io_spawn (const char *path, char * const argv[], unsigned int flags,
     TRACE_LOG1 ("CloseHandle of process failed: ec=%d",
 		(int) GetLastError ());
 
-  for (i = 0; fd_list[i].fd != -1; i++)
-    _gpgme_io_close (fd_list[i].fd);
+  if (! (flags & IOSPAWN_FLAG_NOCLOSE))
+    {
+      for (i = 0; fd_list[i].fd != -1; i++)
+	_gpgme_io_close (fd_list[i].fd);
+    }
 
   for (i = 0; fd_list[i].fd != -1; i++)
     if (fd_list[i].dup_to == -1)
