@@ -361,6 +361,48 @@ _gpgme_get_gpgconf_path (void)
 }
 
 
+const char *
+_gpgme_get_g13_path (void)
+{
+  static char *g13_program;
+
+  LOCK (get_path_lock);
+#if 0
+  if (!g13_program)
+    g13_program = find_program_in_registry ("g13Program");
+#endif
+  if (!g13_program)
+    g13_program = find_program_in_inst_dir ("g13.exe");
+  if (!g13_program)
+    g13_program = find_program_at_standard_place ("GNU\\GnuPG\\g13.exe");
+  UNLOCK (get_path_lock);
+  return g13_program;
+}
+
+
+const char *
+_gpgme_get_uiserver_socket_path (void)
+{
+  static char *socket_path;
+  char *homedir;
+  const char name[] = "S.uiserver";
+
+  if (socket_path)
+    return socket_path;
+
+  homedir = _gpgme_get_default_homedir ();
+  if (! homedir)
+    return NULL;
+
+  socket_path = malloc (strlen (homedir) + 1 + strlen (name) + 1);
+  if (! socket_path)
+    return NULL;
+
+  strcpy (stpcpy (stpcpy (socket_path, homedir), "\\"), name);
+  return socket_path;
+}
+
+
 const char *
 _gpgme_get_w32spawn_path (void)
 {
